// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.6 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/LockStrategy.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:21:25
//
// Module  :  LockStrategy.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.3  WASX.XD   Apr 7, 2005 kjlaw    new code drop
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid;

/**
 * LockStrategy provides an enumerated type idiom for use on the
 * <code>BackingMap.setLockStrategy(LockStrategy)</code> method.
 * It determines whether or not a lock manager is needed for
 * a BackingMap and if so, whether to use a optimistic or
 * pessimistic locking strategy.
 *
 * @since WAS XD 6.0, XC10
 *
 * @see BackingMap#setLockStrategy(LockStrategy)
 */
public final class LockStrategy
{
    /**
     * NONE indicates internal LockManager use is not needed since
     * concurrency control is provided outside of ObjectGrid either by
     * a persistence manager using objectgrid as a side cache,
     * the application, or by a Loader plugin (for example, uses database
     * locks to control concurrency).
     */
    public final static LockStrategy NONE = new LockStrategy((byte) 0);

    /**
     * OPTIMISTIC is typically used for a map that does not
     * have a Loader plugin, the map is read mostly, and
     * locking is neither provided by a persistence manager using
     * ObjectGrid as a side cache or by the application itself.
     * For this strategy, an exclusive lock is obtained on a map
     * entry being inserted, updated, or removed at commit time.
     * The lock ensures version information cannot be changed
     * by another transaction while the transaction being committed
     * is performing an optimistic version check.
     */
    public final static LockStrategy OPTIMISTIC = new LockStrategy((byte) 1);

    /**
     * PESSIMISTIC is typically used for a map that does not
     * have a Loader plugin and locking is neither provided by
     * a persistence manager using ObjectGrid as a side cache, by
     * a Loader plugin, or by the application itself.  It is typically
     * used when optimistic approach fails too often since there
     * are update transactions that frequently collide on the
     * same map entry (e.g. not a read mostly map or large number
     * of clients accessing a small map).
     */
    public final static LockStrategy PESSIMISTIC = new LockStrategy((byte) 2);

    /**
     * unique instance id.
     */
    final private byte ivId;

    /**
     * Constructor
     */
    private LockStrategy()
    {
        super();
        ivId = 0;
    }

    /**
     * Constructor
     * @param id byte value to differentiate different LockStrategy's
     */
    private LockStrategy( byte id )
    {
        super();
        ivId = id;
    }

    /**
     * Returns a string representation of the <code>LockStrategy</code>.
     *
     * @return  a string representation of the <code>LockStrategy</code>.
     */
    public String toString() {
        switch (ivId) {
        case 0: return "NONE LockStrategy"; //$NON-NLS-1$
        case 1: return "OPTIMISTIC LockStrategy"; //$NON-NLS-1$
        case 2: return "PESSIMISTIC LockStrategy"; //$NON-NLS-1$
        default: return "UNKNOWN LockStrategy"; //$NON-NLS-1$
        }
    }
}
