// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
package com.ibm.websphere.objectgrid;

import com.ibm.websphere.objectgrid.plugins.io.DataSerializer;
import com.ibm.websphere.objectgrid.plugins.io.annotations.PluginOutputFormat;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedEntry;


/**
 * Identifies the data output format from data access APIs and plug-ins.
 * @since 8.6
 */
public enum OutputFormat {

    /**
     * <code>NATIVE</code> output format refers to deserialized native Java objects. This is the
     * default output format for maps that do not have data serializer associated, and maps that
     * copy modes other than {@linkplain CopyMode#COPY_TO_BYTES_RAW} is used.<p>
     *
     * When using a custom DataSerializer plug-in, the native format is the object that is returned via the
     * {@link DataSerializer#inflateDataObject(DataObjectContext, XsDataInputStream)} method.
     */
    NATIVE,

    /**
     * <code>RAW</code> output format refers to {@link SerializedEntry} object. It is applicable to maps
     * that are configured with {@link CopyMode#COPY_TO_BYTES} or {@link CopyMode#COPY_TO_BYTES_RAW}.
     * This is the default output format that have data serializer associated, and maps that are
     * configured to use {@link CopyMode#COPY_TO_BYTES_RAW}.
     */
    RAW, 
    
    /**
     * <code>UNDEFINED</code> output format should only be used by {@link PluginOutputFormat} annotation.
     * If it is used, the plug-in will receive cache data in the form specified by the configuration.
     */
    UNDEFINED
}
