// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.6 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/TransactionException.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:25:42
//
// Module  :  TransactionException.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   02/07/2005   sutter    new code drop
// 258492       WASX.XD   05/05/2005   kjlaw     add isTransactionActive and wasTransactionRolledBack methods.
// XD601_M3_B.1 WASX.XD   10/19/2005   kjlaw     all CTORs must have rolledBack parameter to ensure that the
//                          contract for isTransactionActive and wasTransactionRolledBack methods is honored.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid;

/**
 * A general transaction exception indicating something went wrong with a transaction.
 * The <code>isTransactionActive()</code> and <code>wasTransactionRolledBack()</code> methods
 * can be used to determine whether transaction is still active or was rolled back
 * as a result of this exception occuring.
 *
 * @since WAS XD 6.0, XC10
 */
public class TransactionException extends ObjectGridException
{
    /**
     * The <code>serialVersionUID</code> for this class.
     */
    private static final long serialVersionUID = -902780973990770589L;

    /**
     * Indicates whether the transaction was rolled back or not.
     */
    protected boolean ivTransactionRolledBack = false;

    // **************************************************************************
    // **************************************************************************
    // WARNING: do not add other constructors that do not include a boolean
    // rolledBack parameter.  We want to ensure that the contract for
    // wasTransactionRollbedBack method and isTranactionActive methods
    // is honored.  To do this, the boolean rolledBack parameter is needed
    // on every contructor.
    // *************************************************************************
    // **************************************************************************

    /**
     * Returns <code>true</code> if the transaction is active.  Otherwise, <code>false</code>
     * is returned to indicate either the transaction never started or was completed.
     *
     * @return true if the transaction is active
     */
    public boolean isTransactionActive()
    {
        return !ivTransactionRolledBack;
    }

    /**
     * Returns <code>true</code> if the transaction was rolled back.
     *
     * @return true if the transaction was rolled back
     */
    public boolean wasTransactionRolledBack()
    {
        return ivTransactionRolledBack;
    }

    /**
     * Constructs a new TransactionException with the specified detail message and
     * a special indication of whether the transaction was rolled back as a result
     * of this exception. The cause is not initialized, and may subsequently be
     * initialized by a call to the <code>initCause</code> method.
     *
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     * @param rolledBack
     *           A value of <code>true</code> indicates the transaction was rolled back.
     *
     * @see #initCause(Throwable)
     * @see #getMessage()
     * @see #wasTransactionRolledBack()
     */
    public TransactionException(String message, boolean rolledBack)
    {
        super( message );
        ivTransactionRolledBack = rolledBack;
    }

    /**
     * Constructs a new TransactionException with a specified cause and a specified
     * indication of whether the transaction was rolled back as a result of this exception.
     * The cause and a detail message of <code>(cause==null ? null : cause.toString())</code>
     * is used (which typically contains the class and detail message of <code>cause</code>).
     * This constructor is useful for as a wrapper for other Throwable objects that occur.
     *
     * @param cause is the exception that caused this exception to be thrown,
     *        which is saved for later retrieval by the <code>getCause()</code> method.
     *        A <code>null</code> value is permitted and indicates that the cause
     *        is nonexistent or is unknown.
     * @param rolledBack
     *           A value of <code>true</code> indicates the transaction was rolled back.
     *
     * @see #getCause()
     * @see #wasTransactionRolledBack()
     */
    public TransactionException(Throwable cause, boolean rolledBack)
    {
        super( cause );
        ivTransactionRolledBack = rolledBack;
    }

    /**
     * Constructs a new TransactionException with a specified cause and a specified
     * indication of whether the transaction was rolled back as a result of this exception.
     * The cause and a detail message of <code>(cause==null ? null : cause.toString())</code>
     * is used (which typically contains the class and detail message of <code>cause</code>).
     * This constructor is useful for as a wrapper for other Throwable objects that occur.
     *
     * @param cause is the exception that caused this exception to be thrown,
     *        which is saved for later retrieval by the <code>getCause()</code> method.
     *        A <code>null</code> value is permitted and indicates that the cause
     *        is nonexistent or is unknown.
     * @param rolledBack
     *           A value of <code>true</code> indicates the transaction was rolled back.
     *
     * @see #getCause()
     * @see #wasTransactionRolledBack()
     * @since WAS XD 6.1 IFIX1
     */
    public TransactionException(TransactionException cause, boolean rolledBack)
    {
        super( cause );
        ivTransactionRolledBack = rolledBack;
        if (cause != null) {
            cause.ivTransactionRolledBack = rolledBack;
        }
    }

    /**
     * Constructs a new TransactionException with the specified detail message,
     * cause, and indication of whether the transaction was rolled back as a result
     * of this exception.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this TransactionException's detail
     * message.
     *
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). (A<code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown).
     * @param rolledBack
     *           A value of <code>true</code> indicates the transaction was rolled back.
     *
     * @see #getCause()
     * @see #getMessage()
     * @see #wasTransactionRolledBack()
     */
    public TransactionException(String message, Throwable cause, boolean rolledBack)
    {
        super( message, cause );
        ivTransactionRolledBack = rolledBack;
    }

    /**
     * Constructs a new TransactionException with the specified detail message,
     * cause, and indication of whether the transaction was rolled back as a result
     * of this exception.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this TransactionException's detail
     * message.
     *
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). (A<code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown).
     * @param rolledBack
     *           A value of <code>true</code> indicates the transaction was rolled back.
     *
     * @see #getCause()
     * @see #getMessage()
     * @see #wasTransactionRolledBack()
     * @since WAS XD 6.1 IFIX1
     */
    public TransactionException(String message, TransactionException cause, boolean rolledBack)
    {
        super( message, cause );
        ivTransactionRolledBack = rolledBack;
        if (cause != null) {
            cause.ivTransactionRolledBack = rolledBack;
        }
    }
}