// IBM Confidential OCO Source Material
//
// 5724-J34 COPYRIGHT International Business Machines Corp. 2005, 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  Plugin.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.x  WASX.XD   02/23/2005   rajeshk    new code drop
// b896                   05/01/2006   vandersn   open config APIs
//
//
package com.ibm.websphere.objectgrid.config;

import java.io.Serializable;
import java.util.List;

/**
 * <p>This interface represents an ObjectGrid or BackingMap plugin. An <code>ObjectGridConfiguration</code>
 * object supports the following Plugins:</p>
 * <ul>
 *  <LI><code>PluginType#OBJECTGRID_EVENT_LISTENER</code></li>
 *  <LI><code>PluginType#TRANSACTION_CALLBACK</code></li>
 *</ul>
 *
 * <p>A <code>BackingMapConfiguration</code> object supports the following Plugins:</p>
 *<ul>
 *  <LI><code>PluginType#EVICTOR</code></li>
 *  <LI><code>PluginType#MAP_EVENT_LISTENER</code></li>
 *</ul>
 *
 * <p>A Plugin object has following attributes:</p>
 * <ul>
 * <LI>pluginType: the support PluginTypes are listed above</li>
 * <LI>className: the plugin implementation class name. This class name will be used
 * to construct the class object. A default constructor must be implemented. </li>
 * <LI>configProperties: the JavaBean properties for this plugin implementation
 * object.</li>
 * </ul>
 *
 * <p>A plugin object can be created by using <code>ObjectGridConfigFactory.createPlugin(PluginType, String)<code>
 * method. Please refer to <code>ObjectGridConfigFactory</code> for detailed example.</p>
 *
 * @since WAS XD 6.0.1.2, XC10
 */
public interface Plugin extends Serializable {

    /**
	 * Add a ConfigProperty to this object.
	 * @param configProp - ConfigProperty to add to this object
	 */
	public void addConfigProperty(ConfigProperty configProp);

	/**
	 * Set the ConfigProperties for this object
	 * @param configPropList
	 */
	public void setConfigProperties(List /*<ConfigProperty>*/ configPropList);

	/**
     * Get the ConfigProperty objects that have been set on this object.
     * @return a List of the <code>ConfigProperty</code> objects that have been added to this object.
     * @see ConfigProperty
     */
    public List getConfigProperties();

    /**
     * Get the PluginType for this Plugin.
     * @return the PluginType for this Plugin
     */
    public PluginType getPluginType();

    /**
     * Set the <code>PluginType</code> for this Plugin.
     * <p>
     * The <code>ObjectGridConfiguration</code> plugins include
     * <ul>
     *   <li><code>PluginType.TRANSACTION_CALLBACK</code></li>
     *   <li><code>PluginType.OBJECTGRID_EVENT_LISTENER</code></li>
     * </ul>
     *
     * The <code>BackingMapConfiguration</code> plugins include
     * <ul>
     *   <li><code>PluginType.EVICTOR</code></li>
     *   <li><code>PluginType.MAP_EVENT_LISTENER</code></li>
     * </ul>
     * @param pluginType
     */
    public void setPluginType(PluginType pluginType);

    /**
     * Get the String representation of the class name of this Plugin
     * @return the String representation of the class name
     */
    public String getClassName();

    /**
     * The class name that is set must be an implementor of the PluginTypeImpl for this Plugin.  For example,
     * if the type of this Plugin is <code>PluginType.EVICTOR<code>, then the
     * className must be an implementor of the <code>com.ibm.websphere.objectgrid.plugins.Evictor<code> interface.
     * @param className - the class name of the Class that implements the PluginType
     */
    public void setClassName(String className);

    /**
     * Get the OSGi service name configured for this Plugin.
     * If an OSGi service name is configured for this Plugin, the className
     * configured for this Plugin is ignored.
     *
     * @return the OSGi service name configured for this Plugin.
     * @since 7.1.1
     */
    public String getOSGiService();

    /**
     * Set the OSGi service name configured for this Plugin.
     * If an OSGi service name is configured for this Plugin, the className
     * configured for this Plugin is ignored.
     *
     * @param osgiService the OSGi service name configured for this Plugin.
     *
     * @since 7.1.1
     */
    public void setOSGiService(String osgiService);
}

