// IBM Confidential OCO Source Material
//
// 5724-J34 COPYRIGHT International Business Machines Corp. 2005, 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  PluginType.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason    Version   Date         Userid    Change Description
// --------- --------- ------------ --------- -----------------------------------------
// b896                05/01/2006   vandersn  open config APIs
// b24011    7.1.1     06/20/2011   jhanders  Update to add override plug-ins for 7.1.1
// --------- --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.config;

import java.io.Serializable;

import com.ibm.websphere.objectgrid.plugins.BackingMapLifecycleListener;
import com.ibm.websphere.objectgrid.plugins.Evictor;
import com.ibm.websphere.objectgrid.plugins.MapEventListener;
import com.ibm.websphere.objectgrid.plugins.ObjectGridEventListener;
import com.ibm.websphere.objectgrid.plugins.ObjectGridLifecycleListener;
import com.ibm.websphere.objectgrid.plugins.TransactionCallback;
import com.ibm.websphere.objectgrid.plugins.io.MapSerializerPlugin;
import com.ibm.websphere.objectgrid.revision.CollisionArbiter;
import com.ibm.ws.objectgrid.config.PluginTypeImpl;


/**
 * Every <code>Plugin</code> has a PluginType. The PluginType is specified during Plugin creation.
 * <p>
 * <code>ObjectGridConfiguration</code> objects support the following PluginTypes for overriding client-side ObjectGrid plugins:
 * <ul>
 * <li><code>PluginType.TRANSACTION_CALLBACK</code></li>
 * <li><code>PluginType.OBJECTGRID_EVENT_LISTENER</code></li>
 * <li><code>PluginType.COLLISION_ARBITER</code></li>
 * </ul>
 *
 * <p>
 * <code>BackingMapConfiguration</code> objects support the following PluginTypes for overriding client-side BackingMap plugins:
 * <ul>
 * <li><code>PluginType.EVICTOR</code></li>
 * <li><code>PluginType.MAP_EVENT_LISTENER</code></li>
 * </ul>
 *
 * @since WAS XD 6.0.1.2, XC10
 *
 * @see ObjectGridConfigFactory#createPlugin(PluginType, String)
 */
public interface PluginType extends Serializable{
    // ObjectGrid PluginTypes
    /**
     * <code>PluginType.OBJECTGRID_EVENT_LISTENER</code> can be used to attach an {@link ObjectGridEventListener} plugin to an ObjectGrid
     */
    public final static PluginType OBJECTGRID_EVENT_LISTENER = new PluginTypeImpl((byte) 3);

    /**
     * <code>PluginType.TRANSACTION_CALLBACK</code> can be used to attach a {@link TransactionCallback} plugin to an ObjectGrid.
     */
    public final static PluginType TRANSACTION_CALLBACK = new PluginTypeImpl((byte) 2);

    /**
     * <code>PluginType.COLLISION_ARBITER</code> can be used to attach a {@link CollisionArbiter} plugin to an ObjectGrid.
     */
    public final static PluginType COLLISION_ARBITER = new PluginTypeImpl((byte) 15);

    /**
     * <code>PluginType.OBJECTGRID_LIFECYCLE_LISTENER</code> can be used to attach an {@link ObjectGridLifecycleListener} plugin to an ObjectGrid
     * @since 7.1.1
     */
    public final static PluginType OBJECTGRID_LIFECYCLE_LISTENER = new PluginTypeImpl((byte) 17);


    // BackingMap PluginTypes
    /**
     * <code>PluginType.EVICTOR</code> can be used to attach an {@link Evictor} to a BackingMap.
     */
    public final static PluginType EVICTOR = new PluginTypeImpl((byte) 7);

    /**
     * <code>PluginType.MAP_EVENT_LISTENER</code> can be used to attach a {@link MapEventListener} to a BackingMap.
     */
    public final static PluginType MAP_EVENT_LISTENER = new PluginTypeImpl((byte) 10);

    /**
     * <code>PluginType.MAP_SERIALIZER_PLUGIN</code> can be used to attach a {@link MapSerializerPlugin} to a BackingMap to serialize keys.
     * @since 7.1.1
     */
    public final static PluginType MAP_SERIALIZER_PLUGIN = new PluginTypeImpl((byte) 16);

    /**
     * <code>PluginType.MAP_LIFECYCLE_LISTENER</code> can be used to attach a {@link BackingMapLifecycleListener} plugin to an BackingMap
     * @since 7.1.1
     */
    public final static PluginType MAP_LIFECYCLE_LISTENER = new PluginTypeImpl((byte) 18);

}
