// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  AndFilter.java
//
// Source File Description:
//
// This class returns true if and only if all filters are true.
package com.ibm.websphere.objectgrid.continuousquery.filter;

import com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryFilter;
import com.ibm.websphere.objectgrid.continuousquery.exception.ContinuousQueryException;


/**
 * This filter returns true if and only if all its child filters are true.
 * 
 * @since 8.6
 */
public class AndFilter extends BinaryLogicalFilter {

    private static final long serialVersionUID = 8246442998497142758L;
    
    public AndFilter() { }

    /**
     * Construct a filter which will pass only if all child filters pass.
     * @param list the list of child filters to be checked
     */
    public AndFilter(ContinuousQueryFilter... list) {
        super(list);
    }

    public boolean filter(FilterContent content)  throws ContinuousQueryException{
        boolean rc = true;
        for (ContinuousQueryFilter f : filters) {
            rc = rc && f.filter(content);
            if (!rc) break;
        }
        return rc;
    }

    /**
     * @see AbstractCQFilter#createString()
     * 
     * @return "(<filter> AND <filter> ...)", 
     *          where <filter> refers to the string representation of {@linkplain AbstractCQFilter}.
     */
    public String createString() {
        return createString("AND");
    }
}
