// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  FilterContent.java
//
// Source File Description:
//
// A data structure to hold the information needed by the filter chain.
package com.ibm.websphere.objectgrid.continuousquery.filter;

import com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryFilter;
import com.ibm.websphere.objectgrid.plugins.ObjectTransformer;
import com.ibm.websphere.objectgrid.plugins.io.MapSerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedKey;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedValue;

/**
 * A wrapper for cache entry objects which is passed to each {@link ContinuousQueryFilter} associated with
 * a continuous query.  The wrapper contains the key and value for the cache entry
 * that needs to be processed by the filter.
 * @param <KeyType>  Type of the key object for the map being queried
 * @param <ValueType> Type of the value object for the map being queried
 * @since 8.6
 */

public interface FilterContent<KeyType,ValueType> {

    /**
     * Retrieves the key of the object to be filtered.
     * <p>
     * For maps using standard java serialization or a custom {@link ObjectTransformer} the return
     * type of this method will be the key object.
     * <p>
     * For eXtreme Data Format maps and maps with a custom {@link MapSerializerPlugin} the return value
     * of this method is a {@link SerializedKey}.  The {@code SerializedKey} allows access to the key in 
     * its serialized form, or its native Java object form.  
     * <p>
     * @return the key or {@link SerializedKey}
     */
    public KeyType getKey();

    /**
     * Retrieves the value of the object to be filtered.
     * <p>
     * For maps using standard java serialization or a custom {@link ObjectTransformer} the return
     * type of this method will be the value object.
     * <p>
     * For eXtreme Data Format maps and maps with a custom {@link MapSerializerPlugin} the return value
     * of this method is a {@link SerializedValue}.  The {@code SerializedValue} allows access to the key in 
     * its serialized form, or its native Java object form.  
     * <p>
     * @return the value or {@link SerializedValue}
     */
    public ValueType getValue();
}
