// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  LTEFilter.java
//
// Source File Description:
//
// This class checks if an attribute is less than or equals to a scalar.
package com.ibm.websphere.objectgrid.continuousquery.filter;

import java.nio.ByteBuffer;

import com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryFilter;
import com.ibm.websphere.objectgrid.continuousquery.exception.ContinuousQueryException;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer;

/**
 * This checks if an attribute is less than or equals to a scalar.
 * @param <AttributeType> Type of the attribute referenced by the attribute path
 * @param <MatchType> Type of the object being compared to
 * @since 8.6
 */
public class LTEFilter<AttributeType,MatchType> extends CompareFilter<Object,Object,AttributeType,MatchType> {

    private static final long serialVersionUID = -1504420214569239954L;
    
    /**
     * Constructor for serialization only.
     */
    public LTEFilter() { }
    
    /**
     * @see CompareFilter#CompareFilter(String, Object)
     */
    public LTEFilter(String attributePath, MatchType object) {
        super(attributePath, object);
    }

    /**
     * @see ContinuousQueryFilter#filter(FilterContent)
     */
    @SuppressWarnings({"rawtypes", "unchecked"})
    public boolean filter(FilterContent content) throws ContinuousQueryException {

    		Object objectToFilter = getAttribute(content, attributePath);
            if(DataSerializer.SpecialValue.NOT_FOUND.equals(objectToFilter)) return false;
            if (objectToFilter != null && objectToFilter instanceof byte[] && objectToCompare !=null && objectToCompare instanceof byte[]) {
                return ByteBuffer.wrap((byte[]) objectToFilter).compareTo(ByteBuffer.wrap((byte[]) objectToCompare)) <= 0;
            }
            if (objectToFilter != null && objectToFilter instanceof Comparable) {
                return ((Comparable)objectToFilter).compareTo(objectToCompare) <= 0;
            }
            
            return false;
    }

    /**
     * @see AbstractCQFilter#createString()
     * 
     * @return "<attributePath> <= <object>",
     *          where <attributePath> refers to the path for accessing a value from the object to be filtered,
     *          and <object> refers to the string representation of the object to be compared with.
     */
    public String createString() {
        return createString("<=");
    }
}
