// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  EQFilter.java
//
// Source File Description:
//
// This class negates the result of a filter.
package com.ibm.websphere.objectgrid.continuousquery.filter;

import com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryFilter;
import com.ibm.websphere.objectgrid.continuousquery.exception.ContinuousQueryException;


/**
 * This class negates the result of a {@code ContinuousQueryFilter}.
 * @since 8.6
 */
public class NotFilter extends AbstractCQFilter<Object,Object,Object,Object> {

    private static final long serialVersionUID = -4781709454489373557L;
    
    ContinuousQueryFilter filter;

    public NotFilter() {
    }

    /**
     * Construct a new filter which will invert the result of its child filter
     * @param f
     * 		the child filter to be evaluated and inverted
     */
    public NotFilter(ContinuousQueryFilter f) {
        this.filter = f;
    }

    /**
     * @see ContinuousQueryFilter#filter(FilterContent)
     */
    public boolean filter(FilterContent content)  throws ContinuousQueryException{
        return !filter.filter(content);
    }

    /**
     * @see AbstractCQFilter#createString()
     * 
     * @return "!<filter>", 
     *          where <filter> refers to the string representation of {@linkplain AbstractCQFilter}.
     */
    public String createString() {
        return "!" + filter.toString();
    }
    
}
