// IBM Confidential OCO Source Material
//
// 5724-J34 COPYRIGHT International Business Machines Corp. 2006, 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  AgentManager.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b4560        XD6.1                  bnewport  DataGrid first drop
// b3787        XD6.1                  bnewport  Asynchronous Replication, WAS integration, HSBC scenario support
// b8161        XD6.1     05/15/2007   cdjohnson Fix javadoc.
// b9168        XD6.1     07/19/2007   cdjohnson Fix javadoc.
//
package com.ibm.websphere.objectgrid.datagrid;

import java.util.Collection;
import java.util.Map;

import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.security.AgentPermission;

/**
 * The AgentManager is the primary interface for submitting {@link MapGridAgent} or {@link ReduceGridAgent}
 * instances to the ObjectGrid.  Every ObjectMap has an AgentManager and is retrieved using
 * the {@link ObjectMap#getAgentManager()} method.
 * <p>
 *
 * When security is enabled, these methods require permission: com.ibm.websphere.objectgrid.security.AgentPermission.
 * The name of the AgentPermission is the full map name, and the actions are the agent implementation
 * classes or packages. Refer to {@link com.ibm.websphere.objectgrid.security.AgentPermission} for more
 * permission details.<p>
 *
 * Agents run in a transaction separate from the client transaction.  Client transactions that are active
 * when invoking the AgentManager methods are rolled-back when any exception occurs.<p>
 *
 * The MapGridAgent instance may include additional state, such as a query string or parameters,
 * to eliminate some entries.
 *
 * @since WAS XD 6.1
 *
 */
public interface AgentManager
{
	/**
	 * Routes the MapGridAgent instance and invokes the
     * {@link MapGridAgent#process(com.ibm.websphere.objectgrid.Session, ObjectMap, Object)}
     * method on each ObjectGrid server partition associated with the input
     * keys and returns each result key and value in a Map.
     * <p>
     *
     * Required Permission: {@link AgentPermission}
     *
	 * @param agent The MapGridAgent instance to invoke on select partitions.
	 * @param keys The collection of keys. Each key determines which ObjectGrid server partition to route and invoke the agent.
	 * @return A Map holding the result for each processed key. A map entry value result may be of the type: {@link EntryErrorValue}
     * if there was an error generating a result.
	 */
	Map callMapAgent(MapGridAgent agent, Collection keys);

	/**
     * Routes the MapGridAgent instance and invokes the
     * {@link MapGridAgent#processAllEntries(com.ibm.websphere.objectgrid.Session, ObjectMap)}
     * method on all ObjectGrid server partitions and returns each result key and value in a map.
     *<p>
     *
     * Required Permission: {@link AgentPermission}
     *
     * @param agent The MapGridAgent instance to invoke on each partition.
     * @return A Map holding the result for each processed key. A map entry value result may be of the type: {@link EntryErrorValue}
     * if there was an error generating a result.
	 */
	Map callMapAgent(MapGridAgent agent);

	/**
	 * This method performs a multi-level reduce operation on data in a map. The ReduceGridAgent instance is routed to and the
     * {@link ReduceGridAgent#reduce(com.ibm.websphere.objectgrid.Session, ObjectMap, Collection)} method
     * is invoked on each ObjectGrid server partition associated with the input keys. The data may be further reduced
     * using the {@link ReduceGridAgent#reduceResults(Collection)} method.
     * <p>
     *
     * Required Permission: {@link AgentPermission}
     *
     * @param agent The ReduceGridAgent instance to invoke on select partitions.
	 * @param keys The collection of keys holding data to reduce.
     * @return The reduced result, or {@link EntryErrorValue}
	 */
	Object callReduceAgent(ReduceGridAgent agent, Collection keys);

	/**
     * This performs performs a multi-level reduce operation on data in a map. The ReduceGridAgent instance is routed and the
     * {@link ReduceGridAgent#reduce(com.ibm.websphere.objectgrid.Session, ObjectMap)} method
     * is invoked on all ObjectGrid server partitions. The data may be further reduced
     * using the {@link ReduceGridAgent#reduceResults(Collection)} method. <p>
     *
     * The ReduceGridAgent instance may include additional state, such as
     * a query string or parameters, to eliminate some entries.
     * <p>
     *
     * Required Permission: {@link AgentPermission}
     *
     *
     * @param agent The ReduceGridAgent instance to invoke.
     * @return The reduced result, or {@link EntryErrorValue}
	 */
	Object callReduceAgent(ReduceGridAgent agent);
}
