/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 */

package com.ibm.websphere.objectgrid.deployment;

import java.net.URL;
import java.util.List;

import com.ibm.websphere.objectgrid.ObjectGridException;
import com.ibm.websphere.objectgrid.server.Server;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;

/**
 * Creates the DeploymentPolicy object representation from ObjectGrid descriptor files which
 * can be used to create ObjectGrid server containers.
 *
 * @see Server#createContainer(DeploymentPolicy)
 * @since WAS XD 6.1 FIX3
 *
 */
public final class DeploymentPolicyFactory {

    /**
     * Creates the DeploymentPolicy for use in creating a container.
     *
     * @param deploymentPolicyXML
     *            The deployment policy xml file resource locator.  If null, a default deployment policy is created based on the objectGridXML.
     * @param objectGridXML
     *            The object grid xml file resource locator.  This parameter is not allowed to be null.
     * @return The DeploymentPolicy object representation derived from the files specified.
     * @throws ObjectGridException
     *             If the files specified are not valid.
	 * @throws IllegalArgumentExeption 
     *             If objectGridXML is null.
     * @see Server#createContainer(DeploymentPolicy)
     */
    public static DeploymentPolicy createDeploymentPolicy(URL deploymentPolicyXML, URL objectGridXML)
            throws ObjectGridException {
        
        if (objectGridXML == null) {
            throw new IllegalArgumentException("The objectGridXML must not be null");
        }
        
        ObjectGridManagerImpl manager = ObjectGridManagerImpl.instance();
        List objectGridConfigList = manager.getObjectGridConfigurations(objectGridXML, true);
        DeploymentPolicy policy = manager.getDeploymentPolicy(deploymentPolicyXML,
                objectGridConfigList, true);

        return policy;
    }
}
