/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 * b9388                  10/01/2007   vandersn  additional zone support
 * ------------ --------- ------------ --------- ---------------------------------------
 */

package com.ibm.websphere.objectgrid.deployment;

import java.io.Serializable;

/**
 * Defines the type of shard for the ShardMappings.  There are 3 ShardTypes: PRIMARY, SYNCHRONOUS_REPLICA, and ASYNCHRONOUS_REPLICA
 * 
 * @see ShardMapping
 *
 * @since WAS XD 6.1 FIX3
 */
public class ShardType implements Serializable  {

    private static final long serialVersionUID = 3009627809085570691L;

    /**
     * Represents a primary shard.
     */
    public final static ShardType PRIMARY = new ShardType((byte) 0);

    /**
     * Represents a synchronous replica shard.
     */
    public final static ShardType SYNCHRONOUS_REPLICA = new ShardType((byte) 1);

    /**
     * Represents an asynchronous replica shard.
     */
    public final static ShardType ASYNCHRONOUS_REPLICA = new ShardType((byte) 2);

    /**
     * Given the raw value of a ShardType, this method returns a ShardType object, or null if the raw
     * value does not match an existing type.  This method is used to deserialize this object.
     * 
     * @param id the raw value of a ShardType
     * @return the ShardType corresponding to the raw input value
     * @since WXS 8.6
     */
    public final static ShardType valueOf(byte id) {
    	try {
    		return PRIVATE_VALUES[id];
    	} catch (Throwable t) {
    		return null;
    	}
    }
    
    private final static ShardType[] PRIVATE_VALUES = new ShardType[] {PRIMARY, SYNCHRONOUS_REPLICA, ASYNCHRONOUS_REPLICA};

    private byte type;



    private ShardType(byte type) {
        this.type = type;
    }

    private ShardType() {
        this((byte) 0);
    }

    /*
     * Required to avoid creating a different object instance each time the object is deserialized.
     */
    private Object readResolve() {
        return PRIVATE_VALUES[type];
    }

    /**
     * Returns a string representation of the <code>ShardType</code>.
     *
     * @return a string representation of the <code>ShardType</code>.
     */
    public String toString() {
        switch (type) {
        case 0:
            return "PRIMARY"; //$NON-NLS-1$
        case 1:
            return "SYNCHRONOUS_REPLICA"; //$NON-NLS-1$
        case 2:
            return "ASYNCHRONOUS_REPLICA"; //$NON-NLS-1$
        default:
            return "UNKNOWN: " + type; //$NON-NLS-1$
        }
    }
//
//    public static void main(String[] args) throws Exception {
//        // Verify serialization
//        System.out.println("Verifying serialization");
//        ByteArrayOutputStream bos = new ByteArrayOutputStream();
//        ObjectOutputStream oos = new ObjectOutputStream(bos);
//        oos.writeObject(ShardType.SYNCHRONOUS_REPLICA);
//        oos.flush();
//        byte[] bytes = bos.toByteArray();
//
//        ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
//        ObjectInputStream ois = new ObjectInputStream(bis);
//        Object o = ois.readObject();
//
//        if(o != ShardType.SYNCHRONOUS_REPLICA)
//            throw new Exception("readResolve failed.");
//        if(!o.equals(ShardType.SYNCHRONOUS_REPLICA))
//            throw new Exception("equals failed");
//        if(o.equals(ShardType.PRIMARY))
//            throw new Exception("equals failed");
//
//        System.out.println(ShardType.PRIMARY);
//        System.out.println(ShardType.SYNCHRONOUS_REPLICA);
//        System.out.println(ShardType.ASYNCHRONOUS_REPLICA);
//
//        System.out.println("done");
//    }
}
