/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 * b9388                  10/01/2007   vandersn  additional zone support
 * ------------ --------- ------------ --------- ---------------------------------------
 */

package com.ibm.websphere.objectgrid.deployment;

import java.util.Set;

/**
 * Defines placement rules with respect to zones.  The ZoneRule defines placement behavior for the shards that
 * are associated with it via ShardMappings.
 * 
 * @see ShardMapping
 * @since WAS XD 6.1 FIX3
 */
public interface ZoneRule {

    /**
     * Get the name of this ZoneRule
     * @return the name of this ZoneRule
     */
    public String getName();

    /**
     * Get the zones that are part of this ZoneRule.  Shards that use this ZoneRule are eligible to be placed
     * within these zones.
     *
     * @return a Set of zones
     */
    public Set getZones();

    /**
     * Add a zone to this ZoneRule
     * @param zoneName name of the zone to be added
     */
    public void addZone(String zoneName);

    /**
     * Set the zones for this Zone Rule.  Shards that use this ZoneRule are eligible to be placed
     * within these zones.
     *
     * @param zones the zones to set for this rule.
     */
    public void setZones(Set zones);
    
    /**
     * If exclusive placement is enabled, then each ShardMapping that uses this ZoneRule will have its shard
     * placed into a different one of the zones defined for this ZoneRule.<p>  
     * 
     * If exclusive placement is disabled, placement behavior is inclusive.  Inclusive placement requires
     * that the shards for each ShardMapping that use this ZoneRule are placed into the same zone.<p>
     * 
     * @return true if the shards are to be placed in different zones, false if shards are to
     * be placed in the same zone (inclusively).
     * @see #setExclusivePlacement(boolean)
     */
    public boolean isExclusivePlacement();
    
    /**
     * Enable or disable exclusive placement within this ZoneRule.  If exclusive placement is enabled, then each ShardMapping that uses this ZoneRule will have its shard
     * placed into a different one of the zones defined for this ZoneRule.<p>
     * 
     * Consider this example.  The primary and a synchronous replica ShardMapping both use the same ZoneRule with exclusive placement enabled.
     * The ZoneRule applies to zone1 and zone2.  Since exclusive placement is enabled, the primary
     * and the synchronous replica must be placed in different zones for each partition .  If a partition's primary is placed in zone1, 
     * the synchronous replica must be placed in zone2.<p>
     * 
     * If exclusive placement is disabled, placement behavior is inclusive.  Inclusive placement requires
     * that the shards for each ShardMapping that use this ZoneRule are placed into the same zone.<p>
     * 
     * Consider this example.  A synchronous and an asynchronous ShardMapping both use the same ZoneRule with exclusive
     * placement disabled.  The ZoneRule applies to zone1 and zone2.  Since placement is inclusive, the synchronous
     * and the asynchronous replica must be placed into the same zone for each partition.  If a partition's synchronous replica
     * is placed into zone1, the asynchronous replica must also be placed in zone1.
     * 
     * Exclusive placement is enabled by default.
     * 
     * @param exclusivePlacement - if set to true, place all shards in different zones.
     * @see #isExclusivePlacement()
     */
    public void setExclusivePlacement(boolean exclusivePlacement);
}
