// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006-2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  EntityTransaction.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b834         6.1       05/22/2006   rajeshk New part.
// ---------------------------------------------------------------------------------------

package com.ibm.websphere.objectgrid.em;

import com.ibm.websphere.objectgrid.Session;

/**
 * An EntityTransaction is associated with an EntityManager and is used
 * to demarcate a transaction.<p>
 *
 * Use the {@link EntityManager#getTransaction()} method to retreive the EntityManager's
 * EntityTransaction instance.<p>
 *
 * Each EntityManager and EntityTransaction are associated with the ObjectGrid
 * <code>Session</code>.  Transaction demarcation can be performed using either
 * the EntityTransaction or ObjectGrid Session.<p>
 *
 * @since WAS XD 6.1
 * @see EntityManager#getTransaction()
 */
public interface EntityTransaction {

    /**
	* Begin a new transaction.
	* @throws IllegalStateException if isActive() is true.
    * @throws PersistenceException if the transaction is unable to start.
    * @see Session#begin()
	*/
	public void begin();

    /**
	* Commit the current transaction, flushing any uncommitted changes in the
    * EntityManager's context to the cache.
	* @throws IllegalStateException if isActive() is false.
	* @throws PersistenceException if the commit fails.
    * @see Session#commit()
	*/
	public void commit();

    /**
	* Roll back the current transaction.
	* @throws IllegalStateException if isActive() is false.
	* @throws PersistenceException if an unexpected error
	* condition is encountered.
    * @see Session#rollback()
	*/
	public void rollback();

    /**
	* Indicate whether a transaction is in progress.
	* @throws PersistenceException if an unexpected error condition is encountered.
    * @see Session#isTransactionActive()
    * @return true if the transaction is currently active.
	*/
	public boolean isActive();

    /**
    * Mark the current transaction so that the only possible
    * outcome of the transaction is for the transaction to be
    * rolled back.
    * @throws IllegalStateException if isActive() is false.
    * @see Session#markRollbackOnly(Throwable)
    */
    public void setRollbackOnly();

    /**
    * Determine whether the current transaction has been marked
    * for rollback.
    * @throws IllegalStateException if isActive() is false.
    * @see Session#isMarkedRollbackOnly()
    */
    public boolean getRollbackOnly();
}
