// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  PersistenceException.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b1593        6.1       07/27/2006   cdjohnson M4 feature additions
// ---------------------------------------------------------------------------------------
package com.ibm.websphere.objectgrid.em;

import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;

/**
 * Thrown by the EntityManager when a problem occurs.  All other exceptions defined in this
 * package are are subclasses of the <code>PersistenceException</code>.
 *
 * All instances of PersistenceException except for instances of <code>NoResultException</code> and
 * <code>NonUniqueResultException</code> will cause the current transaction,
 * if one is active, to be marked for rollback.
 *
 * @since WAS XD 6.1
 */
public class PersistenceException extends ObjectGridRuntimeException {

    private static final long serialVersionUID = 546417439113129299L;

    /**
     * Constructs a new PersistenceException with <code>null</code> as its detail
     * message. The cause is not initialized, and may subsequently be initialized
     * by a call to the <code>initCause</code> method.
     *
     * @see #initCause(Throwable)
     */
    public PersistenceException() {
		super();
	}

    /**
     * Constructs a new PersistenceException with the specified detail message. The
     * cause is not initialized, and may subsequently be initialized by a call to
     * the <code>initCause</code> method.
     *
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     *
     * @see #initCause(Throwable)
     * @see #getMessage()
     */
    public PersistenceException(String message) {
		super(message);
	}

    /**
     * Constructs a new PersistenceException with the specified detail message and
     * cause.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this PersistenceException's detail
     * message.
     *
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). A <code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown.
     *
     * @see #getCause()
     * @see #getMessage()
     */
	public PersistenceException(String message, Throwable cause) {
		super(message, cause);
	}

    /**
     * Constructs a new PersistenceException with a specified cause.
     * The cause and a detail message of <code>(cause==null ? null : cause.toString())</code>
     * is used (which typically contains the class and detail message of <code>cause</code>).
     * This constructor is useful for PersistenceExceptions that are little more
     * than wrappers for other throwables.
     *
     * @param cause is the exception that caused this exception to be thrown,
     *        which is saved for later retrieval by the <code>getCause()</code> method.
     *        A <code>null</code> value is permitted and indicates that the cause
     *        is nonexistent or is unknown.
     *
     * @see #getCause()
     */
	public PersistenceException(Throwable cause) {
		super(cause);
	}

}
