// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006,2009
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  CompositeIndexes.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b13527       XD 6105   01/13/2008   cheng1    composite index support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.em.annotations;

import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * Defines one or more composite indexes to be used with an entity.  When creating a
 * single-attribute index, define a {@link Index} annotation on the entity attribute.
 *
 * In the following example, three indexes are created; two composite indexes named StateCityZip and LastFirst
 * and a single-attribute index over the birthday attribute:<br>
 * <pre>
 * <literal>@Entity</literal>
 * <literal>@CompositeIndexes({</literal>
 *     <literal>@CompositeIndex</literal>(name="StateCityZip", attributeNames="state,city,zip"),
 *     <literal>@CompositeIndex</literal>(name="LastFirst", attributeNames="lastName,firstName")
 * })
 * public class Person {
 *     <literal>@Id</literal> String taxId;
 *     String lastName;
 *     String firstName;
 *     <literal>@Index</literal> Date birthday;
 *     String state;
 *     String city;
 *     int zip;
 * }
 * </pre>
 *
 * @since 7.0
 */
@Target( { TYPE })
@Retention(RUNTIME)
public @interface CompositeIndexes {
    /**
     * The array of CompositeIndex annotations, each defining a composite index.
     * @return the array of CompositeIndex annotations.
     */
    public CompositeIndex[] value();

}
