package com.ibm.websphere.objectgrid.io;
/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * Module  :  XsDataStreamFactory.java
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- -----------------------------------------
 * b24713       7.1.1     9/20/2011    kulack    Created
 */

import java.io.IOException;



/**
 * The XsDataStreamManager is used to manage and create instances of {@link XsDataInputStream}
 * and {@link XsDataOutputStream} objects. For the most part, data streams
 * created by the XsDataStreamManager are allocated by WebSphere eXtreme Scale runtime
 * components. 
 *  
 * @since 7.1.1
 */
public interface XsDataStreamManager {
    /**
     * Create a new {@link XsDataInputStream} using the bytes specified as
     * the data backing. Do not change the data while the stream is using
     * the data. The manager will not copy the data.
     * 
     * @param bytes - The data used to back the XsDataInputStream.
     * @return XsDataInputStream - the stream used to read the data
     * @throws IOException - an unexpected IO error creating the stream
     */
    public XsDataInputStream createInputStream(byte bytes[]) throws IOException;

    /**
     * Create a new {@link XsDataOutputStream}. The type of storage used for the
     * XsDataOutputStream data backing is unspecified. 
     * 
     * Close the XsDataOutputStream when finished, thereby allowing any pooling
     * of underlying data buffers to be reclaimed by the system.
     *  
     * @return XsDataOutputStream - the data output stream
     * @throws IOException - an unexpected IO error creating the stream
     */
    public XsDataOutputStream createOutputStream() throws IOException;
}
