// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Source File Description:
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d11207       6.1.0.3   05/16/2008   jian.tang  OpenJPA Load and Preload
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.jpa;

import java.util.Map;

import com.ibm.websphere.objectgrid.ObjectGrid;

/**
 * This interface is used to allow users plug in their own persistence property map to
 * the {@link JPATxCallback} object.
 * <p>
 * In some cases, users might want to override the persistence properties defined in the
 * default persistence.xml. The {@link #getPersistenceProperties(ObjectGrid, int)} method can
 * be implemented to provide the override property map.
 *
 * @since WAS XD 6.1.0.3
 *
 */
public interface JPAPropertyFactory {

    /**
     * Provide the persistence properties to override those in the persistence.xml.
     * <p>
     * An ObjectGrid instance and its partition identifier are passed to this method so users
     * can get ObjectGrid attributes, such as ObjectGrid name.
     * The partitionId indicates the partition identifier being used by this ObjectGrid instance
     * if the ObjectGrid is a server ObjectGrid.
     * If the ObjectGrid is a local or client ObjectGrid, the partition id is 0.
     * @param og the ObjectGrid instance this {@link JPATxCallback} object is associated with.
     * @param partitionId the partition identifier if the ObjectGrid instance
     * @return the persistence properties
     */
    Map getPersistenceProperties(ObjectGrid og, int partitionId);

}
