// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Source File Description:
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d11146       WASX.XD   04/08/2008   jian.tang Time-based database update
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.jpa.dbupdate.annotation;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import javax.persistence.Column;

/**
 * This annotation specifies the timestamp field of an entity class that serves for
 * the time-based update purpose. The Timestamp value is
 * used to identify the time or sequence when a database backend record was last updated.
 * <p>
 * In some scenarios, it is important for ObjectGrid to know the backend updates after a
 * certain time, so the data in ObjectGrid can be synchronized with the backend.
 * <p>
 * Although the annotation indicates it is a time stamp, the field does not have to be a
 * TimeStamp type, nor does its value have to take a time stamp format. It could be a integer,
 * long, or any type as long as it can be compared by ">=" using the Java Persistence API (JPA)
 * query.
 * <p>
 * For example, in DB2 9.5, you can define a column using the ROW CHANGE TIMESTAMP format as follows:
 * <pre>
 * ROWCHGTS TIMESTAMP NOT NULL
 *      GENERATED ALWAYS
 *      FOR EACH ROW ON UPDATE AS
 *      ROW CHANGE TIMESTAMP
 * </pre>
 * <p>
 * You can then map your entity class attribute to this column using the {@link Column}
 * annotation and indicate it to be used by for time-based updates using
 * InvalidationTimestamp annotation.
 * <pre>
 * <literal>@Column</literal> (name = "ROWCHGTS", updatable=false)
 * <literal>@Timestamp</literal>
 * public Timestamp ts;
 * </pre>
 * <p>
 * In Oracle, there is a pseudo-column <b>ora_rowscn</b> for the system change number of the record.
 * You can use this column for the same purpose. For example,
 * <pre>
 * <literal>@Column</literal> (name="ora_rowscn", updatable=false)
 * <literal>@Timestamp</literal>
 * protected long scn;
 * </pre>
 *
 * <p>
 * Only a single Version property or field should be used per class.
 *
 * @since WAS XD 6.1.0.3
 */
@Target( { FIELD })
@Retention(RUNTIME)
public @interface Timestamp {

}
