// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  QueryManagerMBean.java
//
// Source File Description:
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 13570         XD6105    12-4-2008   bsandman  code drop
// ------------ --------- ------------ --------- -----------------------------------------


package com.ibm.websphere.objectgrid.management;

import com.ibm.websphere.objectgrid.stats.QueryStatsModule;

/**
 * This MBean interface allows a client process to perform operations on and get status from an ObjectGrid Query Manager
 * running in a dynamic environment. The object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=QueryManager,name=&lt;grid name&gt;,mapset=&lt;mapset name&gt;,partition=&lt;partition number&gt;,host=&lt;host&gt;,ogServerName=&lt;server&gt;
 * </pre>
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object name.
 *
 * @since WAS XD 6.1.0.5, XC10
 */
public interface QueryManagerMBean {

	/**
     * Gets the query's plan creation time attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the plan creation time for this query in milliseconds
     *
     * @see #retrieveStatsModule(String)
     * @see QueryStatsModule#getPlanCreationTime(boolean copy)
     */

	double getPlanCreationTime(String query);

	/**
     * Gets the query's execution time attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the execution time for this query in milliseconds
     *
     * @see #retrieveStatsModule(String)
     * @see QueryStatsModule#getQueryExecutionTime(boolean copy)
     */

	double getQueryExecutionTime(String query);

	/**
     * Gets the query's execution count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the execution count for this query
     *
     * @see #retrieveStatsModule(String)
     * @see QueryStatsModule#getQueryExecutionCount(boolean copy)
     */

	double getQueryExecutionCount(String query);

	/**
     * Gets the query's result count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the result count for this query
     *
     * @see #retrieveStatsModule(String)
     * @see QueryStatsModule#getQueryResultCount(boolean copy)
     */

	double getQueryResultCount(String query);

	/**
     * Gets the query's failure count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the failure count for this query
     *
     * @see #retrieveStatsModule(String)
     * @see QueryStatsModule#getQueryFailureCount(boolean copy)
     */

	double getQueryFailureCount(String query);

	/**
     * Gets the <code>QueryStatsModule</code> used to retrieve statistics
     * associated with the specified query String
     *
     * @return an QueryStatsModule for statistics associated with the specified query String
     */

	QueryStatsModule retrieveStatsModule(String query);

}
