// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2010-2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $HeadURL$
// $Id$
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b21871       XS7.1.x   12/06/2010   jian.tang eXtreme Scale OSGi Support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.management;

import java.util.Map;

import javax.management.openmbean.TabularData;

/**
 * An OSGi MBean to query and manage OSGi service rankings for an entire server instance.
 *
 * This MBean is only registered if the server is running in an OSGi environment, and if
 * the eXtreme Scale server bundle is active.
 * @since 7.1.1
 */
public interface ServerOSGiMBean  {

    /**
     * Gets all available rankings for the specified OSGi service.
     * <p>
     * Each CompositeData (row in the TabularData) contains the following items:
     *
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>Ranking</td>
     *     <td>Integer</td>
     *     <td>An available ranking for the specified OSGi service.</td>
     * </tr>
     * </table>
     * <p>
     * Note: If connected to a server that is not running in an OSGi framework, this
     * method will return an empty TabularData object.
     *
     * @param serviceName the OSGi service name
     * @return a JMX tabular data, with each row containing an available ranking.
     */
    TabularData retrieveAllRankings(String serviceName);

    /**
     * Get all available rankings for all the OSGi services
     * <p>
     * This JMX tabular data has a nested inner TabularData.
     * <p>
     * Each outer CompositeData (row in the outer TabularData) contains the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>Service</td>
     *     <td>String</td>
     *     <td>The name of the OSGi service.</td>
     * </tr>
     * <tr>
     *     <td>Rankings</td>
     *     <td>TabularData</td>
     *     <td>An inner TabularData containing all available rankings for an OSGi service.</td>
     * </tr>
     * </table>
     * <p>
     * Each inner CompositeData (row in the inner TabularData) contains the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>Ranking</td>
     *     <td>Integer</td>
     *     <td>An available ranking for the OSGi service in the outer row.</td>
     * </tr>
     * </table>
     * <p>
     * Note: If connected to a server that is not running in an OSGi framework, this
     * method will return an empty TabularData object.
     *
     * @return a JMX tabular data of available service rankings. Each row of the
     * TabularData object is a nested TabularData, with each row containing an
     * available ranking.
     */
    TabularData getAllRankings();

    /**
     * Gets all currently used rankings for the specified OSGi service.
     * <p>
     * Each CompositeData (row in the TabularData) contains the following items:
     *
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>ObjectGrid Name</td>
     *     <td>String</td>
     *     <td>The name of the ObjectGrid instance.</td>
     * </tr>
     * <tr>
     *     <td>MapSet Name</td>
     *     <td>String</td>
     *     <td>The name of the MapSet instance.</td>
     * </tr>
     * <tr>
     *     <td>Ranking</td>
     *     <td>Integer</td>
     *     <td>The currently used OSGi service ranking for that ObjectGrid instance.</td>
     * </tr>
     * </table>
     * <p>
     * Note: If connected to a server that is not running in an OSGi framework, this
     * method will return an empty TabularData object.
     *
     * @param serviceName the OSGi service name
     * @return a JMX tabular data, with each row containing an ObjectGrid name
     * and its currently used service ranking.
     */
    TabularData retrieveAllCurrentlyUsedRankings(String serviceName);

    /**
     * Get currently used rankings for all the OSGi services.
     *
     * <p>
     * This JMX tabular data has a nested inner TabularData.
     * <p>
     * Each outer CompositeData (row in the outer TabularData) contains the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>ObjectGrid Name</td>
     *     <td>String</td>
     *     <td>The name of the ObjectGrid instance.</td>
     * </tr>
     * <tr>
     *     <td>MapSet Name</td>
     *     <td>String</td>
     *     <td>The name of the MapSet instance.</td>
     * </tr>
     * <tr>
     *     <td>Service Rankings</td>
     *     <td>TabularData</td>
     *     <td>An inner TabularData containing all currently-used rankings for an OSGi service.</td>
     * </tr>
     * </table>
     * <p>
     * Each inner CompositeData (row in the inner TabularData) contains the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>Service</td>
     *     <td>String</td>
     *     <td>The OSGi service name.</td>
     * </tr>
     * <tr>
     *     <td>Ranking</td>
     *     <td>Integer</td>
     *     <td>The currently used OSGi service ranking for the ObjectGrid instance in the outer row.</td>
     * </tr>
     * </table>
     * <p>
     * Note: If connected to a server that is not running in an OSGi framework, this
     * method will return an empty TabularData object.
     * @return a JMX tabular data of all currently used service rankings. Each row of the
     * TabularData object is a nested TabularData for an OSGi service, with each row
     * containing the name of the ObjectGrid and it's currently used service ranking.
     */
    TabularData getAllCurrentlyUsedRankings();

    /**
     * Checks if the specified OSGi service rankings are available or not.
     * <p>
     * This method takes a Map<String, Integer> parameter so it might not be
     * enabled by certain MBean client console versions, for example,
     * JConsole shipped by Java 5.0. Use the {@link #checkServiceAvailability(String, Integer)}
     * method instead.
     * <p>
     * Each CompositeData (row in the TabularData) contains the following items:
     *
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>Service</td>
     *     <td>String</td>
     *     <td>The OSGi service name.</td>
     * </tr>
     * <tr>
     *     <td>Unavailable Ranking</td>
     *     <td>Integer</td>
     *     <td>The service ranking that the users wants to check and is not available. </td>
     * </tr>
     * </table>
     * <p>
     * Note: If connected to a server that is not running in an OSGi framework, this
     * method will return an empty TabularData object.
     *
     * @param services a map of service rankings. The key of the map is the
     * service name, and the value is a collection of available rankings for that
     * service.
     * @return an empty TabularData if the specified service rankings are all available; otherwise
     * a map that contains unavailable service rankings.
     */
    TabularData retrieveServiceAvailability(Map<String, Integer> services);

    /**
     * Checks if the specified OSGi service and ranking is available or not.
     *
     * @param service the OSGi service name
     * @param ranking the OSGi service ranking
     * @return true if the specified service ranking is available; otherwise false.
     */
    boolean checkServiceAvailability(String service, Integer ranking);
}
