// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $HeadURL$
// $Id$
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b21871       XS7.1.1   01/11/2011   jian.tang eXtreme Scale OSGi Support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.osgi;

import java.util.Collection;
import java.util.Map;

/**
 * The eXtreme Scale runtime tracks all the services that are configured as user plug-ins.
 * An OSGiServiceRepository object tracks these services and their service rankings that are
 * available to the eXtreme Scale runtime in this process, and the service rankings in use.
 * <p>
 *
 * Users can use {@link com.ibm.websphere.objectgrid.osgi.OSGiManagerFactory#getOSGiServiceRepository()}
 * method to get an instance of OSGiServiceRepository.
 *
 *
 * @since 7.1.1
 *
 */
public interface OSGiServiceRepository {

    /**
     * Get all available service rankings for the OSGi service.
     * @param serviceName the OSGi service name
     * @return a collection of all available service rankings for that service.
     */
    Collection<Integer> getAllRankings(String serviceName);

    /**
     * Get all available service rankings for all the OSGi services
     *
     * @return a map of available service rankings. The key of the map is the
     * service name, and the value is a collection of available service rankings
     * for that service.
     */
    Map<String, Collection<Integer>> getAllRankings();

    /**
     * Get all currently used service rankings for the specified OSGi service.
     *
     * @param serviceName the OSGi service name
     * @return A map of the currently used service rankings. The key of the map
     * is the name of the ObjectGrid name and MapSet that uses the service, and the value
     * of the map is the service ranking being used.
     */
    Map<OSGiServiceKey, Integer> getCurrentlyUsedRankings(String serviceName);

    /**
     * Get currently used service rankings for all the OSGi services.
     *
     * @return a map of currently used service rankings for all the services.
     * The key of the map is the ObjectGrid and MapSet name, and the value of is a sub-map, where
     * the key of the sub-map is the service name, and the value
     * of is the service ranking being used.
     */
    Map<OSGiServiceKey, Map<String, Integer>> getAllCurrentlyUsedRankings();

    /**
     * Check if the specified service rankings are available or not.
     *
     * @param serviceRankings a map of service rankings. The key of the map is the
     * service name, and the value is a collection of rankings for that
     * service.
     * @return null if the specified service rankings are all available; otherwise
     * a map that contains unavailable service rankings.
     */
    Map<String, Integer> retrieveServiceAvailability(Map<String, Integer> serviceRankings);


    /**
     * The name of the entities that each OSGi Service is scoped to.
     * @since 7.1.1
     */
    public static class OSGiServiceKey {
        private final String ogName;
        private final String msName;

        /**
         * Retrieve the name of the ObjectGrid.
         * @return the ObjectGrid name
         */
        public String getObjectGridName() {
            return ogName;
        }

        /**
         * Return the name of the MapSet
         * @return the MapSet name
         */
        public String getMapSetName() {
            return msName;
        }

        public OSGiServiceKey(String ogName, String msName) {
            super();
            this.ogName = ogName;
            this.msName = msName;
        }

        @Override
        public int hashCode() {
            final int prime = 31;
            int result = 1;
            result = prime * result + ((msName == null) ? 0 : msName.hashCode());
            result = prime * result + ((ogName == null) ? 0 : ogName.hashCode());
            return result;
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (obj == null)
                return false;
            if (getClass() != obj.getClass())
                return false;
            OSGiServiceKey other = (OSGiServiceKey) obj;
            if (msName == null) {
                if (other.msName != null)
                    return false;
            } else if (!msName.equals(other.msName))
                return false;
            if (ogName == null) {
                if (other.ogName != null)
                    return false;
            } else if (!ogName.equals(other.ogName))
                return false;
            return true;
        }

        @Override
        public String toString() {
            StringBuilder builder = new StringBuilder();
            builder.append("OSGiServiceKey [");
            if (ogName != null)
                builder.append("ogName=").append(ogName).append(", ");
            if (msName != null)
                builder.append("msName=").append(msName);
            builder.append("]");
            return builder.toString();
        }
    }

}
