// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.6 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/EvictionEventCallback.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:22:24
//
// Module  :  EvictionEventCallback.java
//
// Source File Description: 
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   01/14/2005   sutter    new code drop
// XD60_M2_B.2  WASX.XD   03/21/2005   sutter    M2 Deliverable
// 269516       WASX.XD   04/25/2005   sutter    change eviction processing to use standard programming model
// 281081       WASX.XD   06/07/2005   kjlaw     add get/put evictor data methods.
// d328136      WASX.XD   12/08/2005   kjlaw     externalize EvictorData.
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins;

import java.util.List;

import com.ibm.websphere.objectgrid.ObjectGridException;

/**
 * <p>An instance of <code>EvictionEventCallback</code> is passed into the 
 * <code>Evictor</code> at initialization time. When an eviction method is 
 * called, corresponding methods of <code>EvictionEventCallback</code> will be 
 * called so the <code>BackingMap</code> can process evictions.</p>
 * 
 * @since WAS XD 6.0, XC10
 * 
 * @see Evictor
 * @see EvictorData
 */
public interface EvictionEventCallback 
{    
    /**
     * Sets the evictor data for a specified BackingMap cache key. 
     * <p>
     * This method can be used by an implementor of the <code>Evictor</code>
     * interface to keep data that the evictor needs for determining which 
     * cache entry to evict.
     * 
     * @param key is the key for accessing a BackingMap entry.
     * @param data the EvictorData object to store as evictor data for a specified key.
     * 
     * @throws IllegalArgumentException if key is a null reference or there
     *        is no BackingMap cache entry for this key.
     * 
     * @see Evictor
     * @since WAS XD 6.0.1
     */
    void setEvictorData(Object key, EvictorData data); 

    /**
     * Gets the evictor data for a specified BackingMap cache entry. 
     * 
     * @param key  the key for the BackingMap entry to set.
     * 
     * @return  if the specified key is not found in BackingMap, then the 
     *          special value <code>EvictorData.KEY_NOT_FOUND</code> is 
     *          returned. If the key is found in the BackingMap, the same 
     *          reference that was previously passed to the 
     *          <code>setEvictorData(Object, EvictorData)</code> method of this
     *          interface is returned.  A null reference is returned if the key 
     *          is found, but the <code>setEvictorData</code> method was not 
     *          previously called for the specified key. 
     * 
     * @throws IllegalArgumentException if key is a null reference.
     * 
     * @see #setEvictorData(Object, EvictorData)
     * @see EvictorData#KEY_NOT_FOUND
     * @since WAS XD 6.0.1
     */ 
    EvictorData getEvictorData(Object key); 

    /**
     * This method is the preferred method for the <code>Evictor</code> to use 
     * when evicting map entries. A list of <code>EvictorData</code> objects is
     * passed as an argument to this method.  For each <code>EvictorData</code>
     * object in the list, the key is obtained from the <code>EvictorData</code>
     * object and used to determine which BackingMap entry to evict.  The 
     * BackingMap entry is evicted if and only if the cache entry for BackingMap 
     * entry contains the exact same <code>EvictorData</code> object in it.  That 
     * is, the java == operator is used to ensure it is the exact same 
     * <code>EvictorData</code> object.  If the == operator indicates a different 
     * object, then the map entry is not evicted.  For those map entries that are 
     * physically evicted from the map, the <code>Evictor</code> will receive 
     * notification through its <code>apply</code> method.
     * 
     * @param evictorDataList a list of <code>EvictorData</code> objects to process.  
     *                        The caller must guarantee this parameter is not 
     *                        <code>null</code> or contain any <code>null</code>
     *                        references.
     * 
     * @throws ObjectGridException if an error occurs during processing
     * @throws java.lang.ClassCastException if an object in evictorDataList does
     *         not implement the <code>EvictorData</code> interface.
     *     
     * 
     * @see Evictor#apply(LogSequence)
     * @see EvictorData#getKey() 
     * @since WAS XD 6.0.1
     */
    void evictMapEntries(List /*<EvictorData>*/ evictorDataList) throws ObjectGridException;

    /**
     * If an <code>Evictor</code> chooses not to implement the 
     * <code>EvictorData</code> interface, this method can be used to evict a 
     * map entry.  However, the <code>Evictor</code> must be prepared to handle
     * the exposure of an application removing and recreating a map entry
     * before the <code>Evictor</code> has an opportunity to call this method.
     * <p>
     * For this method, a list of map keys is passed.  The list is evaluated
     * and an eviction is conducted on the list. When the entries are physically
     * evicted from the map, the <code>Evictor</code> will receive notification 
     * through its <code>apply</code> method.
     * 
     * @param keysToEvictList List of keys to evict from the map.
     *                        The caller must guarantee this parameter is not 
     *                        <code>null</code> or contain any <code>null</code>
     *                        references.
     * 
     * @throws ObjectGridException if an error occurs during processing
     * 
     * @see Evictor#apply(LogSequence)
     */
    void evictEntries(List /*<Object key>*/ keysToEvictList) throws ObjectGridException;
}
