// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id
//
// Module  :  ReplicationMapListener.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b9732        6.1       09/12/2007   cdjohnson Rename replciationStarts to replicationStarts
// b10090       XD 6.1    10/10/2007   cdjohnson Fix javadoc
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins;

import com.ibm.websphere.objectgrid.ClientReplicableMap;

/**
 *
 * This interface is used to create an implementation of an event listener
 * for client-side maps that are in replication mode. Registered listeners
 * receive notification callbacks for replication start and exit events and data changes.<p>
 *
 * Listener instances can be registered with a map using the
 * {@link ClientReplicableMap#enableClientReplication(com.ibm.websphere.objectgrid.ClientReplicableMap.Mode, int[], ReplicationMapListener)} method.
 *
 * @since WAS XD 6.1, XC10
 * @deprecated The client replicated map function is deprecated in version 8.6.  Use the {@link com.ibm.websphere.objectgrid.continuousquery.ContinuousQueryManager} function.
 */
public interface ReplicationMapListener {

    /**
     * This method is invoked when a snapshot mode replica has been synchronized
     * with the client or a continuous replica has started replicating.<p>
     *
     * See also: {@link ClientReplicableMap#enableClientReplication(com.ibm.websphere.objectgrid.ClientReplicableMap.Mode, int[], ReplicationMapListener)}
     */
    void replicationStarts();

    /**
     * This method is invoked when a data change is replicated to the client replication map.
     *
     * @param logSequence the log sequence containing all of the data changes.
     */
    void onData(LogSequence logSequence);

    /**
     * This method is invoked when replication has been disabled for this map.
     * @see ClientReplicableMap#disableClientReplication()
     */
    void replicationExits();
}
