// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2010
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  LFUEvictor.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   02/03/2005   sutter    new code drop
// XD60_M1_B.12 WASX.XD   04/27/2005   sutter    Make Eviction (LRU/LFU) processing work
// 272056       WASX.XD   05/03/2005   jhanders  combine changes() and evictees()
// XD60_M4_B.21 WASX.XD   05/23/2005   sutter    Client/server optimizations
// 279591       WASX.XD   05/28/2005   kjlaw     performance/scalability cleanup.
// 287285       WASX.XD   06/28/2005   kjlaw     make changes required as new BaseMapLogSequence updates.
// 287677       WASX.XD   06/29/2005   kjlaw     use new EvictorData interfaces.
// 294637.1     WASX.XD   08/04/2005   kjlaw     implement RollbackEvictor interface.
// XD601_M3_B.x WASX.XD   10/28/2005   kjlaw     add getNumberOfLRUQueues method.
// d328136      WASX.XD   12/08/2005   kjlaw     externalize EvictorData.
// d332063      WASX.XD   12/13/2005    kjlaw     destroy needs to use doPriv
// d335510      WASX.XD   01/04/2006   jitang    Use AccessController.doPrivileged
// d337865.1    WXD601.XD 02/12/2006   kjlaw     apply needs to handle incorrect order better
// d347038      WXD601.XD 02/14/2006   kjlaw     Need to maintain EvictorData even when map is not activated.
// d350934      WXD601.XD 02/28/2006   kjlaw     clear BinaryHeaps on deactivate.
// bug 11228    6.1.0.2   02/07/2008   jhanders   Update Evictor to not output IllegalArgumentException
// bug 11502    6.1.0.3   03/14/2008   cheng1    enable memory based eviction.
// bug  12770   6.1.0.3   07/03/2008   cheng1    allow unlimited size indicates by maxSize=0 when useMaxMemory=true.
// bug 18905    7.1       04/23/2010   bsandman  added disk delegator abstraction
// b18690       7.1       05/06/2010   jhanders  Update to not use reflection
// bug 18690    7.1       05/14/2010   jhanders  Code review comments for Evictor disk offload implementation
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins.builtins;

import java.lang.reflect.Method;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.OutputFormat;
import com.ibm.websphere.objectgrid.plugins.CacheEntry;
import com.ibm.websphere.objectgrid.plugins.EvictionEventCallback;
import com.ibm.websphere.objectgrid.plugins.Evictor;
import com.ibm.websphere.objectgrid.plugins.EvictorData;
import com.ibm.websphere.objectgrid.plugins.LogElement;
import com.ibm.websphere.objectgrid.plugins.LogSequence;
import com.ibm.websphere.objectgrid.plugins.RollbackEvictor;
import com.ibm.websphere.objectgrid.plugins.io.annotations.PluginOutputFormat;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.DiffMapValue;
import com.ibm.ws.objectgrid.ObjectGridImpl;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;
import com.ibm.ws.objectgrid.Storage;
import com.ibm.ws.objectgrid.io.offheap.LFUOffHeapEvictionData;
import com.ibm.ws.objectgrid.io.offheap.LFUOffHeapEvictionDelegator;
import com.ibm.ws.objectgrid.io.offheap.ObjectGridHashTableOH;
import com.ibm.ws.objectgrid.io.offheap.OffHeapEvictionData;
import com.ibm.ws.objectgrid.io.offheap.OffHeapManager;
import com.ibm.ws.objectgrid.io.offheap.impl.XsOffHeapEvictionDataImpl;
import com.ibm.ws.objectgrid.management.MemoryEvictionController;
import com.ibm.ws.objectgrid.management.MemoryEvictionControllerImpl;
import com.ibm.ws.objectgrid.management.MemoryManagement;
import com.ibm.ws.objectgrid.management.MemoryManagementImpl;
import com.ibm.ws.objectgrid.map.BaseMap;
import com.ibm.ws.objectgrid.plugins.DiskLFUEvictorData;
import com.ibm.ws.objectgrid.plugins.EvictionDelegator;
import com.ibm.ws.objectgrid.plugins.LFUEvictionDelegator;
import com.ibm.ws.objectgrid.plugins.LFUEvictorData;
import com.ibm.ws.objectgrid.plugins.LFUEvictorDataTransformer;
import com.ibm.ws.objectgrid.plugins.MemoryEvictionTrigger;
import com.ibm.ws.objectgrid.plugins.OffheapEntry;
import com.ibm.ws.objectgrid.plugins.SystemCacheEntry;
import com.ibm.ws.objectgrid.util.BinaryHeap;
import com.ibm.ws.objectgrid.util.DiskHelper;
import com.ibm.ws.objectgrid.util.DiskHelperFactory;
import com.ibm.ws.objectgrid.util.ObjectGridUtil;
import com.ibm.ws.xs.NLSConstants;
import com.ibm.ws.xs.thread.ScheduledTask;
import com.ibm.ws.xs.thread.ThreadPool;
import com.ibm.ws.xs.thread.ThreadPoolManagerFactory;
import com.ibm.ws.xs.util.XSUtilities;

/**
 * This class manages a BackingMap using a simple Least Frequently Used (LFU) algorithm. It attempts to keep the
 * BackingMap at less than a certain number of entries based on a usage count of each entry. This class uses an array of
 * binary heap objects for keeping a EvictorData object created for a CacheEntry. The EvictorData object has the LFU
 * count and key for the CacheEntry. The idea is to spread cache entries across multiple binary heap objects so that
 * there are fewer CacheEntry objects that collide on a synchronization point since they all do not use the same binary
 * heap object.
 * <p>
 * An evictor thread is spawned during initialization that wakes up periodically and processes the array of binary heap
 * objects to determine if any CacheEntry needs to be evicted. The idea of using a binary heap object is to obtain an
 * ordering by LFU value so that it is not necessary to enumerate over every CacheEntry to determine if it needs to be
 * evicted. Since binary heap is a partial ordering, the cost to do the ordering is cheaper than creating a totally
 * ordered list and/or sorting the list. But it also means the LFU algorithm is not 100% accurate, but it is close
 * enough to be useful and avoids the cost of being 100% accurate by keep a totally ordered list.
 *
 * @since WAS XD 6.0, XC10
 *
 * @see Evictor
 */
@PluginOutputFormat(keyFormat=OutputFormat.RAW, valueFormat=OutputFormat.RAW)
final public class LFUEvictor implements Evictor, RollbackEvictor, Runnable {
    private final static String CLASS_NAME = LFUEvictor.class.getName();

    private final static TraceComponent tc = Tr.register(CLASS_NAME, Constants.TR_EVICTOR_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);

    /**
     * Default sleep time for evictor thread if the <code>setSleepTime(int)</code> method is not called.
     *
     * @see #setSleepTime(int)
     */
    public static final long DEFAULT_SLEEP_TIME = 15000L; // 15 seconds

    /**
     * Default number of binary heaps to create if the <code>setNumberOfHeaps(int)</code> method is not called.
     *
     * @see #setNumberOfHeaps(int)
     */
    public static final int DEFAULT_NUMBER_OF_HEAPS = 51;

    /**
     * Root comparable object to use in each BinaryHeap object created.
     */
    private final static LFUEvictorData ROOT_COMPARABLE = new LFUEvictorData();

    private static final Method triggerEvictionMethod;

    static {
        try {
            triggerEvictionMethod = (Method) AccessController.doPrivileged(new PrivilegedExceptionAction() {

                public Object run() throws Exception {
                    Method m = LFUEvictor.class.getDeclaredMethod("triggerEviction", new Class[0]);
                    m.setAccessible(true);
                    return m;
                }

            });
        } catch (Exception e) {
            IllegalStateException ise = new IllegalStateException(
                    "Failed to initialize class due to following exception");
            ise.initCause(e);
            throw ise;
        }
    }

    /** The EvictionEventCallback instance */
    EvictionEventCallback ivCallback;

    /** Reference to the BackingMap */
    BackingMap ivBackingMap;

    private int ivNumberOfHeaps = DEFAULT_NUMBER_OF_HEAPS;

    private BinaryHeap[] ivHeaps;

    private int ivMaxSize = 0;

    private long ivSleepTime = DEFAULT_SLEEP_TIME;

    private MemoryManagement memoryManagement;

    private MemoryEvictionTrigger listener;

    private boolean ivUseMemoryUsageThresholdEviction = false;

    private volatile boolean ivQuit = true;

    private boolean ivLogInvalidLogElement = true;

    private boolean ivDestroyed = false;

    private boolean ivActivated = false;

    private String evictorId = " ";

    private ScheduledTask scheduledEviction;

    private Thread activeThread;

    private final ThreadPool threadPool = ThreadPoolManagerFactory.getThreadPoolManager().getThreadPool("LFUEvictor");

    private ClassLoader classLoader;

    private static boolean hasOutputUsageThresholdNotSupportedMsg = false;

    private MemoryEvictionController memEvictionController;

    private boolean ivUseMemoryEvictionController = false;

    private boolean disk = false;

    private boolean offheap = false;

    private LFUEvictionDelegator delegator = null;

    private HeapCount[] heapCounts = null;

    private int divisor = 0;

    private static class HeapCount {
        int count = 0;
    }

    /**
     * Creates a LFUEvictor object with default values for the maximum size per heap, the number of heaps, and sleep
     * time between sweeps of the heaps by the evictor thread.
     * <p>
     * The default values can be overridden by use of the <code>setMaxSize(int)</code>,
     * <code>setNumberOfHeaps(int)</code>, and <code>setSleepTime(int)</code> methods. If the <code>setMaxSize</code>
     * method is never called, the size of the map is unlimited.
     *
     * @see #DEFAULT_NUMBER_OF_HEAPS
     * @see #DEFAULT_SLEEP_TIME
     * @see #setMaxSize(int)
     * @see #setNumberOfHeaps(int)
     * @see #setSleepTime(int)
     */
    public LFUEvictor() {
        super();
    }

    /**
     * Gets the maximum size of each binary heap.
     *
     * @return the same value that was passed to the <code>setMaxSize(int)</code> method or the default value of zero if
     *         the <code>setMaxSize</code> method is never called
     *
     * @see #setMaxSize(int)
     */
    public int getMaxSize() {
        return ivMaxSize;
    }

    /**
     * @see Evictor#destroy()
     */
    synchronized public void destroy() {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "destroy: " + evictorId);
        }
        ivDestroyed = true;
        deactivate(ObjectGridImpl.DESTROY_DEFAULT);
    }

    synchronized private void destroy(int destroyType) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "destroy: " + ObjectGridImpl.getDestroyType(destroyType) + " " + evictorId);
        }
        ivDestroyed = true;
        deactivate(destroyType);
    }

    /**
     * @see Evictor#initialize(BackingMap, EvictionEventCallback)
     */
    public void initialize(BackingMap map, EvictionEventCallback callback) {
        evictorId = "LFUEvictorForMap-" + map.getName() + "_" + map.getPartitionId();
        classLoader = Thread.currentThread().getContextClassLoader();
        ivBackingMap = map;
        ivCallback = callback;
        
        BaseMap baseMap = ((BaseMap) ivBackingMap);
        
        disk = baseMap.getStorage() == Storage.DISK;
        offheap = baseMap.getStorage() == Storage.OFF_HEAP;

        int numPartitions = map.getPartitionManager().getNumOfPartitions();
        if (numPartitions > 1 && numPartitions == ivNumberOfHeaps) {
            divisor = ivNumberOfHeaps;
        }

        // if disk skip Queue creation and memory initialization...will take out all memory processing in flow
        if (disk || offheap) {
        	baseMap.setEvictorDataTransformer(LFUEvictorDataTransformer.instance);

            if (disk) {
                DiskHelper diskHelper = DiskHelperFactory.getDiskHelper();
                delegator = (LFUEvictionDelegator) diskHelper.createEvictionDelegator(EvictionDelegator.LFU_DERBY,
                        ivNumberOfHeaps);
            } else /*offheap*/{
                delegator = new LFUOffHeapEvictionDelegator();
            }

            try {
                delegator.initialize(map, evictorId);

            } catch (Throwable e) {
                FFDCFilter.processException(e, CLASS_NAME + ".init", "286", this);
                throw new RuntimeException(e);
            }

            heapCounts = new HeapCount[ivNumberOfHeaps];
            for (int i = 0; i < ivNumberOfHeaps; i++) {
                heapCounts[i] = new HeapCount();
            }

            try {
                int[] hcs = delegator.getHeapCounts();
                for (int i = 0; i < this.ivNumberOfHeaps; i++) {
                    synchronized (heapCounts[i]) {
                        heapCounts[i].count = hcs[i];
                    }
                }
            } catch (Throwable e) {
                FFDCFilter.processException(e, CLASS_NAME + ".initialize", "299");
            }
        } else {

            // get BackingMap's trigger.
            String evictionTriggers = ivBackingMap.getEvictionTriggers();
            if (evictionTriggers != null) {
                // parse the evictionTriggers to see if memory eviction is specified.
                ivUseMemoryUsageThresholdEviction = ObjectGridUtil.isTriggerInEvictionTriggers(
                        BackingMap.EVICTIONTRIGGER_MEMORY_USAGE_THRESHOLD, evictionTriggers);

                if (ivUseMemoryUsageThresholdEviction) {
                    ivUseMemoryEvictionController = false;
                } else {
                    ivUseMemoryEvictionController = ObjectGridUtil.isTriggerInEvictionTriggers(
                            MemoryEvictionController.MEMORY_USAGE_MONITOR_TRIGGER, evictionTriggers);
                }
            }

            // Size unlimited?
            if (ivMaxSize <= 0 && !ivUseMemoryUsageThresholdEviction) {
                // Yep, no need to create any heaps.
                ivHeaps = new BinaryHeap[0];
                ivNumberOfHeaps = 0;
            } else {
                // Nope, need to create the heaps.
                ivHeaps = new BinaryHeap[ivNumberOfHeaps];

                // Create BinaryHeap objects for the TTLCompartor objects created
                // for each CacheEntry in BackingMap.
                for (int i = 0; i < ivNumberOfHeaps; ++i) {
                    ivHeaps[i] = new BinaryHeap(ROOT_COMPARABLE);
                }
            }
        }

        if (ivUseMemoryUsageThresholdEviction) {
            memoryManagement = MemoryManagementImpl.instance();
        }

        if (ivUseMemoryEvictionController) {
            this.memEvictionController = MemoryEvictionControllerImpl.instance();
        }

    }

    /**
     * @see Evictor#apply(LogSequence)
     */
    public void apply(LogSequence sequence) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "apply");
        }

        if (ivMaxSize > 0 || ivUseMemoryUsageThresholdEviction) {
            // Process each LogElement in LogSequence to determine if TTLCompartor
            // needs to be added, updated, or removed from BinaryHeap.
            Iterator iter = sequence.getAllChanges();

            // Apply method is called after transaction has completed and locks are released.
            // Thus it is possible that the order apply method is called does not match
            // the order the transaction completed in. For example, a INSERT tran followed
            // by a UPDATE tran might call this method in the reverse order (UPDATE followed
            // by INSERT). Thus we can no depend on LogElement type by itself to determine
            // how to process the LogElement. Instead, we must examine whether key exists in
            // BackngMap and if it does whether the BackingMap entry contains the EvictorData.
            for (int i = sequence.size() - 1; i >= 0; --i) {
                try {
                    LogElement elem = (LogElement) iter.next();
                    CacheEntry entry = ((DiffMapValue)elem).getRawCacheEntry();
                    Object key = elem.getKey();
                    LogElement.Type type = elem.getType();
                    if (type == LogElement.INSERT) {
                        EvictorData evictorData = ivCallback.getEvictorData(key);
                        if (evictorData == null) {
                            // Key does exist in backing map, but CacheEntry for the key
                            // does not contain any EvictorData object. So create and
                            // insert EvictorData into BinaryHeap and into the CacheEntry
                            // for the key in BackingMap.
                            insertLFU(key, entry);
                        } else if (evictorData != EvictorData.KEY_NOT_FOUND) {
                            // Key does exist in BackingMap and the CacheEntry for the key contains
                            // the EvictorData. So update the LFU data.
                            updateLFU(key, entry, evictorData);
                            
                            // Offheap: If we succeeded in our get of the evictor data from
                            //  an offheap hashtable, we need to release the GET pin placed
                            //  on it.
                            if(entry instanceof OffheapEntry)
                            	OffHeapManager.getInstance().unpinXsEvictionData(((OffHeapEvictionData)evictorData).getAddress(), OffHeapManager.JAVA_GET_PIN);
                        }
                    } else if (type == LogElement.UPDATE || type == LogElement.FETCH || type == LogElement.TOUCH) {
                        EvictorData evictorData = ivCallback.getEvictorData(key);
                        if (evictorData == null) {
                            // Key does exist in backing map and the CacheEntry for the key
                            // does not contain any EvictorData object. So create and
                            // insert EvictorData into BinaryHeap and into the CacheEntry
                            // for the key in BackingMap.
                            insertLFU(key, entry);
                        } else if (evictorData != EvictorData.KEY_NOT_FOUND) {
                            // Key does exist in BackingMap and the CacheEntry for the key contains
                            // the EvictorData. So update the LFU data.
                            updateLFU(key, entry, evictorData);
                        } else {
                            // Key does not exist in BackingMap, so do nothing. This can happen if
                            // tran 1 does a INSERT and tran 2 and UPDATE, but apply for each tran
                            // is called in reverse order. If that happens, then when INSERT tran
                            // calls apply we will insert the LFU data into queue at that time.
                        }
                    } else if (type == LogElement.DELETE || type == LogElement.EVICT) {
                        // getEvictorData will do what is below.  Being an internal plug-in
                        // we can do it faster.
                        /*EvictorData evictorData = ivCallback.getEvictorData(key);*/
                        DiffMapValue dmv = (DiffMapValue) elem;
                        SystemCacheEntry beforeImage = (SystemCacheEntry) dmv.getRawBeforeImage();
                        EvictorData evictorData = (beforeImage == null) ? null : beforeImage.getEvictorData();
                        
                        if (evictorData != null && evictorData != EvictorData.KEY_NOT_FOUND) {
                            removeLFU(elem, entry, evictorData);
                        }
                    } else if (type == LogElement.CLEAR) {
                        if (disk || offheap) {
                            delegator.truncateTable();
                            for (int idx = 0; idx < ivNumberOfHeaps; ++idx) {
                                synchronized (heapCounts[idx]) {
                                    heapCounts[idx].count = 0;
                                }
                            }
                        } else if ((ivHeaps != null) && (ivNumberOfHeaps > 0)) {
                            for (int idx = 0; idx < ivNumberOfHeaps; ++idx) {
                                synchronized (ivHeaps[idx]) {
                                    ivHeaps[idx] = new BinaryHeap(ROOT_COMPARABLE);
                                }
                            }
                        }
                    } else {
                        // unknown type, dump information, and continue...
                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc, "Unknown type of LogElement: " + type);
                        }

                        if (ivLogInvalidLogElement) {
                            // Only log the first time this happens.
                            ivLogInvalidLogElement = false;

                            // CWOBJ0012E: The LogElement type code, {0} ({1}), is not recognized for this operation.
                            Integer code = new Integer(type.getCode());
                            String string = type.toString();
                            Tr.error(tc, NLSConstants.INVALID_LOGELEMENT_TYPE_CWOBJ0012, new Object[] { code, string });
                        }
                    }
                } catch (Throwable t) {
                    // We do not expect this to ever happen, but if it does, we simply FFDC and
                    // log warning message. We will continue with the loop so that we have a
                    // chance to process the rest of the LogElements in the LogSequence.
                    FFDCFilter.processException(t, CLASS_NAME + ".apply", "326");
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "apply caught unexpected Throwable", t);
                    }
                    Tr.warning(tc, NLSConstants.IGNORING_UNEXPECTED_EXCEPTION_CWOBJ0002,
                            new Object[] { t, "LFUEvictor" });
                }
            }
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "apply");
        }
    }

    /**
     * @param key
     *            the key of the entry
     * @return the BinaryHeap to use for a specified CacheEntry.
     */
    private BinaryHeap getBinaryHeap(CacheEntry entry) {
        // Hash the key to determine which BinaryHeap to use
        // for this CacheEntry. This should give us a good distribution
        // across the BinaryHeap.
        int hash = determineEntryHeap(entry);
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "getBinaryHeap using heap in bucket " + hash);
        }
        BinaryHeap heap = ivHeaps[hash];
        return heap;
    }

    private int determineEntryHeap(CacheEntry entry) {
        int hashCode = entry.hashCode();
        return XSUtilities.computeHash(hashCode, divisor) % ivNumberOfHeaps;
    }

    /**
     * Insert LFU EvictorData object into BinaryHeap for a specified key and CacheEntry. This method MUST be called only
     * when key exists in BackingMap and CacheEntry does NOT contain any EvictorData object.
     *
     * @param key
     *            is the key of BackingMap entry.
     * @param cacheEntry
     *            is the CacheEntry that currently does NOT contain any EvictorData object.
     */
    private void insertLFU(Object key, CacheEntry cacheEntry) throws Exception {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "insertLFU for key: " + key);
        }

        // Create LFUComparable for this key and save in BackingMap.
        LFUEvictorData comparable;
        if (disk) {
            comparable = new DiskLFUEvictorData(key);
        } else if (offheap) {
            ObjectGridHashTableOH oht = (ObjectGridHashTableOH) ((BaseMap) ivBackingMap).getCacheEntries();
            XsOffHeapEvictionDataImpl xsEvictData = oht.createXsOffHeapEvictionData(key, cacheEntry);
            comparable = new LFUOffHeapEvictionData(OffHeapManager.getInstance(), oht.getXsOffHeapMapImpl(), xsEvictData); //Offheap: allocation pin
        } else {
            comparable = new LFUEvictorData(key);
        }
        boolean entryInserted = true;
        try {
            ivCallback.setEvictorData(key, comparable);
        } catch (IllegalArgumentException iae) {
            entryInserted = false;
        }

        if ((disk || offheap) && entryInserted) {
            int heap = determineEntryHeap(cacheEntry);
            HeapCount count = heapCounts[heap];
            synchronized(count){
            	delegator.insertData(comparable, key, cacheEntry, heap);
            	count.count++;
            }
            if (offheap) 
            	OffHeapManager.getInstance().unpinXsEvictionData(((LFUOffHeapEvictionData) comparable).getXsOffHeapEvictionData().getAddress(), OffHeapManager.JAVA_GET_PIN);
        } else if (entryInserted) {
            // Get the LFUComparable from the CacheEntry slot reserved for evictor to use.
            BinaryHeap heap = getBinaryHeap(cacheEntry);

            // Now insert LFUComparable into the BinaryHeap for this CacheEntry.
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, "inserting LFUComparable for key: " + comparable.getKey() + " in heap " + heap);
            }

            synchronized (heap) {
                heap.insert(comparable);
            }
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "insertLFU for key: " + key);
        }
    }

    /**
     * This method is used to update the LFU data when the key exists in BackingMap AND the CacheEntry does contain the
     * EvictorData object for the key.
     *
     * @param key
     * @param cacheEntry
     * @param evictorData
     */
    private void updateLFU(Object key, CacheEntry cacheEntry, EvictorData evictorData) throws Exception {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "updateLFU for key: " + key);
        }
        LFUEvictorData comparable = (LFUEvictorData) evictorData;
        if (disk) {
            int heap = determineEntryHeap(cacheEntry);
            delegator.updateData(comparable, key, cacheEntry, heap);
        }else if (offheap) {
            int heap = determineEntryHeap(cacheEntry);
            LFUOffHeapEvictionData data = (LFUOffHeapEvictionData) evictorData;
            delegator.updateData(data, key, cacheEntry, heap);
        } else {
            // Get the TTLComparable from the CacheEntry slot reserved for evictor to use.
            BinaryHeap heap = getBinaryHeap(cacheEntry);

            // Key exists in map and CacheEntry contains a LFUComparable.

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, "updating LFUComparable for key: " + comparable.getKey()
                        + ", count prior to increment  = " + comparable.getCount());
            }

            // Remove LFUComparable from heap, increment LFU counter, and insert
            // back into the heap with new counter value so that heap is reordered.
            synchronized (heap) {
                heap.delete(comparable);
                comparable.incrementCount();
                heap.insert(comparable);
            }
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "updateLFU for key: " + key);
        }
    }

    /**
     * Remove the LFUComparable object from the BinaryHeap that is associated with a specified LogElement of type DELETE
     * or EVICT.
     *
     * @param elem
     *            is a LogElement where elem.getType is LogElement.DELETE or LogElement.EVICT.
     */
    private void removeLFU(LogElement elem, CacheEntry cacheEntry, EvictorData evictorData) throws Exception {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "removeLFU for evictorData: " + evictorData);
        }
        if (disk || offheap) {
            int heap = determineEntryHeap(cacheEntry);
            HeapCount count = heapCounts[heap];
            synchronized(count){
                delegator.removeData(evictorData, heap);
                if (count.count > 0) count.count--;
            }
        } else {
                // Yes there is. Remove from the BinaryHeap since CacheEntry
                // no longer exists.
            LFUEvictorData comparable = (LFUEvictorData) evictorData;
            BinaryHeap heap = getBinaryHeap(cacheEntry);
            synchronized (heap) {
                heap.delete(comparable);
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, "removed LFUComparable for key: " + comparable.getKey() + " from heap " + heap);
            }
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "removeLFU");
        }
    }

    /**
     * @see RollbackEvictor#rollingBack(LogSequence)
     */
    public void rollingBack(LogSequence sequence) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "rollingBack");
        }

        // Process each LogElement in LogSequence and update
        // expiration time only if TTLData exists for the key.
        Iterator iter = sequence.getAllChanges();
        for (int i = sequence.size() - 1; i >= 0; --i) {
            try {
                // Get next LogElement.
                LogElement elem = (LogElement) iter.next();

                if (elem.getType() != LogElement.CLEAR) {
                    // Get the evictor data from the CacheEntry.
                    CacheEntry cacheEntry = ((DiffMapValue)elem).getRawCacheEntry();
                    Object key = elem.getKey();
                    EvictorData evictorData = ivCallback.getEvictorData(key);
                    if (evictorData != null && evictorData != EvictorData.KEY_NOT_FOUND) {
                        // key does exist in BackingMap and CacheEntry contains EvictorData,
                        // so update the LFU data for this key.
                        updateLFU(key, cacheEntry, evictorData);
                    }
                }
            } catch (Throwable t) {
                FFDCFilter.processException(t, CLASS_NAME + ".rollback", "747", this);
                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                    Tr.debug(tc, "rollback caught unexpected Throwable", t);
                }
                Tr.warning(tc, NLSConstants.IGNORING_UNEXPECTED_EXCEPTION_CWOBJ0002, new Object[] { t, "LFUEvictor" });
            }
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "rollingBack");
        }
    }

    /**
     * Gets number of binary heaps being used.
     *
     * @return the same value that was passed to the <code>setNumberOfHeaps(int)</code> method or the default value of
     *         <code>DEFAULT_NUMBER_OF_HEAPS</code> if the <code>setNumberOfHeaps</code> method is never called
     *
     * @see #DEFAULT_NUMBER_OF_HEAPS
     * @see #setNumberOfHeaps(int)
     */
    public int getNumberOfHeaps() {
        return ivNumberOfHeaps;
    }

    /**
     * Sets the number of binary heaps to use.
     * <p>
     * This method is used to override the default number of binary heaps created by the <code>initialize</code> method
     * for keeping the usage count data needed by the evictor thread. This method must be called prior to the
     * <code>initialize</code> method to avoid an IllegalStateException being thrown. If this method is not called,the
     * <code>DEFAULT_NUMBER_OF_HEAPS</code> constant is used as the number of heaps.
     *
     * @param numberOfHeaps
     *            is the number of BinaryHeap instances used to hold entry usage count data. The value must be greater
     *            than or equal to 1 and a prime number is recommended for best performance.
     *
     * @throws IllegalArgumentException
     *             if numberOfHeaps < 1
     * @throws IllegalStateException
     *             if called after the initialize method.
     *
     * @see #DEFAULT_NUMBER_OF_HEAPS
     * @see #initialize(BackingMap, EvictionEventCallback)
     */
    public void setNumberOfHeaps(int numberOfHeaps) {
        if (numberOfHeaps < 1) {
            throw new IllegalArgumentException("numberOfHeaps argument must be > 0");
        }

        if (ivHeaps != null) {
            throw new IllegalStateException("must be called prior to initialize method.");
        }

        ivNumberOfHeaps = numberOfHeaps;
    }

    /**
     * Gets the sleep time being used in seconds.
     *
     * @return the same value that was passed to the <code>setSleepTime(int)</code> method or the default value of
     *         <code>DEFAULT_SLEEP_TIME</code> if the <code>setSleepTime</code> method is never called
     *
     * @see #setSleepTime(int)
     */
    public int getSleepTime() {
        return (int) (ivSleepTime / 1000L);
    }

    /**
     * Sets the sleep time to use in seconds.
     * <p>
     * This method is used to override the default sleep time of the evictor thread in seconds. This method must be
     * called prior to the <code>initialize</code> method to avoid an IllegalStateException being thrown. If this method
     * is not called, the <code>DEFAULT_SLEEP_TIME</code> constant is used as the sleep time.
     *
     * @param seconds
     *            is the number of seconds the evictor thread sleeps in between each sweep of the binary heap data being
     *            kept for entry usage count data.
     *
     * @throws IllegalArgumentException
     *             if seconds < 1
     * @throws IllegalStateException
     *             if called after the initialize method.
     *
     * @see #DEFAULT_SLEEP_TIME
     * @see #initialize(BackingMap, EvictionEventCallback)
     */
    public void setSleepTime(int seconds) {
        if (seconds < 1) {
            throw new IllegalArgumentException("seconds argument must be > 0");
        }

        if (ivHeaps != null) {
            throw new IllegalStateException("must be called prior to initialize method.");
        }

        ivSleepTime = seconds * 1000L;
    }

    /**
     * Sets the maximum size of each binary heap.
     * <p>
     * This method is used to override the default maximum size for each heap used to keep entry usage count data. The
     * evictor thread will attempt to keep each heap to be no larger than the maximum size. This method must be called
     * prior to the <code>initialize</code> method to avoid an IllegalStateException being thrown.
     *
     * @param maxSize
     *            is the maximum size per heap. Any value <= 0 indicates to allow each heap to be of unlimited size. In
     *            which case, no entry usage count data is kept.
     *
     * @throws IllegalStateException
     *             if called after the initialize method.
     *
     * @see #initialize(BackingMap, EvictionEventCallback)
     */
    public void setMaxSize(int maxSize) {
        if (ivHeaps != null) {
            throw new IllegalStateException("must be called prior to initialize method.");
        }

        ivMaxSize = maxSize;
    }

    private void doMemoryEviction() {
        double recommendedPercentToEvict = memEvictionController.getEvictionRecommendation();

        BinaryHeap heap;
        long totalSize = 0;

        for (int i = 0; i < ivNumberOfHeaps; ++i) {
            try {
                heap = ivHeaps[i];
                List evictList = null;

                synchronized (heap) {
                    int size = heap.size();
                    long targetSize = (long) (size - recommendedPercentToEvict * size);
                    if (ivMaxSize > 0 && ivMaxSize < targetSize) {
                        targetSize = ivMaxSize;
                    }

                    if (size > targetSize) {
                        evictList = new ArrayList((int) (size - targetSize));
                        while (heap.size() > targetSize) {
                            evictList.add(heap.deleteMin());
                        }
                    }
                    totalSize += heap.size();
                }
                if (evictList != null && evictList.size() > 0) {
                    ivCallback.evictMapEntries(evictList);
                }
            } catch (Throwable t) {
                // Should never happen, but let's make sure evictor does not die
                FFDCFilter.processException(t, CLASS_NAME + ".doMemoryEviction", "973");
                if (tc.isEventEnabled()) {
                    Tr.event(tc, evictorId + " caught unexpected Throwable: " + t, t);
                }

                // CWOBJ0002W: ObjectGrid component is ignoring unexpected exception: {0}.
                Tr.warning(tc, NLSConstants.IGNORING_UNEXPECTED_EXCEPTION_CWOBJ0002, new Object[] { t, "LFUEvictor" });
            }
        }
    }

    /**
     * Periodically wakes up and evicts entries.
     *
     * @see Runnable#run()
     */
    public void run() {
        synchronized (this) {
            scheduledEviction = null;
            if (activeThread == null) {
                activeThread = Thread.currentThread();
            } else {
                return;
            }
        }

        Thread currentThread = Thread.currentThread();
        String currentThreadName = currentThread.getName();
        currentThread.setContextClassLoader(classLoader);
        currentThread.setName(currentThreadName + ": " + evictorId);

        try {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, "run called by thread: " + evictorId + ", countHandleManagementEvent = "
                        + (listener == null ? 0 : listener.getMemoryEvictionCount()));
            }

            // Loop until destroy or deactivate is called or we break out of the loop.
            while (!ivQuit) {

                if (ivUseMemoryEvictionController) {
                    doMemoryEviction();

                    // original code path is intact in else block
                } else {
                    // when ivUseMemoryUsageThresholdEviction is true,
                    // after wake up, need to check if it is notified by MemoryNotification event.
                    // if YES, need to evict entries even the current size has not exceeded max size.
                    boolean isUsageThresholdExceeded = false;
                    if (ivUseMemoryUsageThresholdEviction) {
                        isUsageThresholdExceeded = memoryManagement.isUsageThresholdExceeded();
                    }
                    // Process data in array of BinaryHeap objects.
                    int totalSize = 0;
                    boolean traceSize = ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled();
                    for (int i = 0; i < ivNumberOfHeaps; ++i) {
                        try {
                            List evictList = null;
                            if (disk || offheap) {
                                //get heap size from memory instead of delegator.getTableSize();
                                HeapCount count = heapCounts[i];
                                synchronized (count) {
                                    long tableSize = count.count;
                                    // Figure out num to evict
                                    long numToEvict = 0;
                                    if (ivMaxSize > 0) {
                                        numToEvict = (tableSize > ivMaxSize) ? tableSize - ivMaxSize : 0;
                                    }
                                    if (numToEvict > 0) evictList = delegator.getEvictList(i, numToEvict);
                                }

                                if (evictList != null && !evictList.isEmpty()) {
                                    if (tc.isDebugEnabled()) {
                                        Tr.debug(tc, "LFUEvictor evicting: " + evictList);
                                    }
                                    ivCallback.evictMapEntries(evictList);
                                    
                                    // Offheap: Release the pins from the getEvictList call
                                    if(offheap) {
                                    	for(OffHeapEvictionData ohed : (List<OffHeapEvictionData>)evictList) {
                                    		OffHeapManager.getInstance().unpinXsEvictionData(ohed.getAddress(), OffHeapManager.JAVA_GET_PIN);
                                    	}
                                    }
                                }

                            } else {
                                BinaryHeap heap = ivHeaps[i];
                                processEvictorHeap(heap, isUsageThresholdExceeded);
                                if (traceSize) {
                                    synchronized (heap) {
                                        totalSize += heap.size();
                                    }
                                }
                            }
                        } catch (Throwable t) {
                            // Should never happen, but let's make sure evictor does not die
                            // unexpectantly by tolerating all Throwable objects. Just log
                            // the fact that it occured.
                            FFDCFilter.processException(t, CLASS_NAME + ".run", "493", evictorId);
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc, evictorId + " caught unexpected Throwable: " + t, t);
                            }

                            // C0002W: ObjectGrid component is ignoring unexpected exception: {0}.
                            Tr.warning(tc, NLSConstants.IGNORING_UNEXPECTED_EXCEPTION_CWOBJ0002, new Object[] { t,
                                    "LFUEvictor" });
                        }
                    }

                    if (traceSize) {
                        Tr.debug(tc, evictorId + " total number of LFUComparable objects in heap = " + totalSize);
                    }
                }

                // return to original code path for sleep logic
                try {
                    // Sleep for a while before sweeping over BinaryHeap list.
                    // Thread.sleep( ivSleepTime );

                    boolean goToSleep = true;
                    if (ivUseMemoryUsageThresholdEviction) {
                        // Need to yield a bit here, or we may never give the VM a chance
                        // to update the usage count, which means we'd evict everything.
                        Thread.sleep(100);

                        // before go to waiting state, ensure isUsageThresholdExceeded is false when
                        // ivUseMemoryUsageThresholdEviction
                        // if the map size is 0 or we're no longer exceeding the usage threshold, finish and reschedule for later (goToSleep)
                        goToSleep = (((BaseMap)ivBackingMap).size() == 0 || !memoryManagement.isUsageThresholdExceeded());
                    }

                    if (goToSleep) {
                        // break out of the while loop and schedule the next eviction
                        break;
                    }
                } catch (InterruptedException e) {
                    // FFDCFilter.processException(t, CLASS_NAME + ".run", "353", threadName);
                    // ignore exception.
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, evictorId + " was interrupted");
                    }
                }
            }

            synchronized (this) {
                if (activeThread == Thread.currentThread()) {
                    activeThread = null;
                }
                if (!ivQuit) {
                    scheduledEviction = threadPool.schedule(this, ivSleepTime);
                } else {
                    if (ivDestroyed) {
                        if (disk || offheap) {
                            delegator.dropTable();
                            heapCounts = null;
                        }
                        ivHeaps = null;
                        ivNumberOfHeaps = 0;
                    }
                }
                notifyAll();
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, evictorId + " terminating, countHandleManagementEvent = "
                        + (listener == null ? 0 : listener.getMemoryEvictionCount()));
            }
        } finally {
            currentThread.setName(currentThreadName);
        }
    }

    // this method is never called in this class, but is called via reflection from MemoryEvictionTrigger
    // this method must remain private it is a external class
    private synchronized void triggerEviction() {
        if (!ivQuit && activeThread == null) {
            // Cancel the scheduledEviction if there is one
            if (scheduledEviction != null) {
                scheduledEviction.cancel();
            }
            threadPool.execute(this);
        }
    }

    /**
     * Process an Enumeration for a BinaryHeap of TTLCompartor objects.
     *
     * @param heap
     *            is the BinaryHeap to process.
     */
    private void processEvictorHeap(BinaryHeap heap, boolean isUsageThresholdExceeded) {
        // Ensure heap has at least 1 element to process
        // Note, size does not include the ROOT object passed
        // to BinaryHeap constructor since the ROOT should never
        // be returned.
        Enumeration e;
        int heapSize = 0;
        synchronized (heap) {
            heapSize = heap.size();
            if (!isUsageThresholdExceeded && ivMaxSize == 0) {
                // not in critical memory usage and ivMaxSize == 0 (unlimited size)
                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                    Tr.debug(tc, "processEvictorHeap for heap of size: " + heapSize + ", isUsageThresholdExceeded: "
                            + isUsageThresholdExceeded + ", ivMaxSize: " + ivMaxSize + ", No need to evict");
                }
                return;
            }

            if (heapSize <= 0) {
                return;
            }
            e = heap.elements();
        }

        /*
         * Enumerate elements until we have evicted enough cache entries to bring this heap down to max size per heap.
         * The heap is ordered from lowest to highest LFU count, so the first n in the heap are the ones to evict. The
         * heap is a partial ordering, so we may not necessary evict the all of the lowest LFU, but it should be close
         * enough.
         */
        int numberToEvict = 0; // (heapSize > ivMaxSize) ? heapSize - ivMaxSize : 0;
        if (ivMaxSize > 0) {
            // normal mode
            numberToEvict = (heapSize > ivMaxSize) ? heapSize - ivMaxSize : 0;
        }
        // else, the numberToEvict = 0; when UsageThresholdExceeded mode or ivMaxSize==0

        if (numberToEvict == 0 && isUsageThresholdExceeded) {
            // even the heap size has not reached max size, but the memory exceeded event has occurred,
            // still need to evict entries to avoid OutOfMemory error.
            // evict 3% for now.
            // to do: find a better logic to determine the number to evict for various situations
            double temp = heapSize * 0.03;
            numberToEvict = (int) temp;

            if (numberToEvict == 0) {
                // there is not many entries, less than 34.
                if (heapSize >= 20) {
                    numberToEvict = 3;
                } else if (heapSize >= 10) {
                    numberToEvict = 2;
                } else {
                    numberToEvict = 1;
                }
            }
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "processEvictorHeap for heap of size: " + heapSize + ", isUsageThresholdExceeded: "
                    + isUsageThresholdExceeded + ", ivMaxSize: " + ivMaxSize + ", numberToEvict: " + numberToEvict);
        }

        List evictList = new ArrayList();
        while (e.hasMoreElements() && (numberToEvict > 0)) {
            LFUEvictorData c = (LFUEvictorData) e.nextElement();
            if (c != null) {
                if (c == ROOT_COMPARABLE) {
                    // This is not expected to ever occur according to Andrew Wilkinson,
                    // but we will tolerate it if it does occur.
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "BinaryHeap nextElement returned root, which is not expected to occur");
                    }
                } else {
                    evictList.add(c);
                    synchronized (heap) {
                        heap.delete(c);
                    }
                    --numberToEvict;
                }
            }
        }

        // If above loop added any key to the key list to evict, then use
        // the EvictorCallback to request the cache entries to be evicted.
        // Note, an emply list is possible since multiple threads calling apply
        // method and the apply method may have removed some of the
        // BinaryHeap entries. This may have occured after Enumeration was
        // created, so keyList may contain keys of entries that no longer
        // exist. That is okay since the evict operation will treat this
        // as a no-op for this key.
        try {
            if (!evictList.isEmpty()) {
                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                    Tr.debug(tc, "LFUEvictor evicting: " + evictList);
                }
                ivCallback.evictMapEntries(evictList);
            }
        } catch (Throwable ex) {
            // FFDCFilter.processException(ex, CLASS_NAME + ".processEvictorHeap", "635", this );

            // No Throwable are expected, but we are going to tolerate any that does
            // occur to ensure that this loop continues to process every key in the
            // eviction list.
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "processEvictorHeap evicted " + evictList.size() + " cache entries from heap of size "
                    + heapSize);
        }
    }

    /**
     * @see Evictor#activate()
     */
    synchronized public void activate() {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "activate: " + evictorId + ", MemoryUsageThresholdEviction: "
                    + ivUseMemoryUsageThresholdEviction);
        }

        if (!ivActivated) {
            ivActivated = true;
            // always spin new thread if ivUseMemoryUsageThresholdEviction is true.
            if (ivQuit && (ivMaxSize > 0 || ivUseMemoryUsageThresholdEviction)) {
                ivQuit = false;

                // spawn evictor thread
                threadPool.execute(this);
            }

            if (!disk && !offheap) {
                if (!ivQuit && ivUseMemoryUsageThresholdEviction) {
                    if (!memoryManagement.isUsageThresholdSupported()) {
                        if (!hasOutputUsageThresholdNotSupportedMsg) {
                            // there might be thoushands evictors activate and deactivate
                            // only out put warning once, it will show something like:
                            // LFUEvictor W CWOBJ0035W: Memory utilization threshold not supported for this JVM.
                            Tr.warning(tc, NLSConstants.MEMORY_UTILIZATION_THRESHOLD_NOT_SUPPORTED_CWOBJ0035);
                            hasOutputUsageThresholdNotSupportedMsg = true;
                        }
                        ivUseMemoryUsageThresholdEviction = false;
                    } else {
                        // register listener with MemoryManagement

                        listener = new MemoryEvictionTrigger(triggerEvictionMethod, this, "LFU");
                        memoryManagement.addEventHandler(listener);
                    }
                }

                // new memory eviction. This cannot be enabled if the old mechanism is enabled
                if (ivUseMemoryEvictionController && !ivQuit) {
                    listener = new MemoryEvictionTrigger(triggerEvictionMethod, this, "LFU");
                    memEvictionController.addEventHandler(this, this.ivBackingMap, listener);
                }
            }
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "activate");
        }
    }

    /**
     * @see Evictor#deactivate()
     */
    synchronized public void deactivate() {
        deactivate(ObjectGridImpl.DESTROY_DEFAULT);
    }

    synchronized private void deactivate(int destroyType) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "deactivate: " + ObjectGridImpl.getDestroyType(destroyType) + " " + evictorId);
        }

        if (!ivQuit) {
            ivQuit = true;
            // Cancel the scheduledEviction if there is one
            if (scheduledEviction != null) {
                scheduledEviction.cancel();
            }
        }

        if (listener != null) {
            if (ivUseMemoryUsageThresholdEviction) {
                // remove from MemoryManagement
                memoryManagement.removeEventHandler(this.listener);
            } else if (this.ivUseMemoryEvictionController) {
                memEvictionController.removeEventHandler(this, this.ivBackingMap, this.listener);
            }
        }

        ivActivated = false;
        if (activeThread != null) {
            try {
                wait();
            } catch (InterruptedException e) {
                // fine
            }
        }

        if (ivDestroyed) {
            // Deactivate called by destroy method, so destroy array
            // of BinaryHeap that holds the evictor data if evictor
            // thread is not running. If evictor thread is running,
            // let the evictor thread destroy binary heap objects
            // before the thread terminates.
            if (ivQuit) {
                // if this is a DefaultGrid map, don't drop the table, we don't do that for DefaultGrid so we
                // can hibernate the data.
                if ((disk || offheap) && destroyType != ObjectGridImpl.DESTROY_MEMORY_ONLY
                        && (ivBackingMap == null || (!ivBackingMap.getObjectGrid().getName().equals("DefaultGrid") &&
                        						     !ivBackingMap.getObjectGrid().getName().equals("_IBM_XC10_CONFIG_"))
                            )) {
                    delegator.dropTable();
                    heapCounts = null;
                }
                ivHeaps = null;
                ivNumberOfHeaps = 0;
            }
        } else {
            // We are deactivating, but not destroying LFUEvictor. We need
            // to clear the BinaryHeap of data since the BaseMap was cleared
            // of all cache entries as part of the deactivate.
            if (disk || offheap) {
                delegator.truncateTable();
                for (int i = 0; i < ivNumberOfHeaps; i++) {
                    synchronized (heapCounts[i]) {
                        heapCounts[i].count = 0;
                    }
                }
            } else {
                if ((ivHeaps != null) && (ivNumberOfHeaps > 0)) {
                    for (int i = 0; i < ivNumberOfHeaps; ++i) {
                        ivHeaps[i] = new BinaryHeap(ROOT_COMPARABLE);
                    }
                }
            }
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "deactivate");
        }
    }
}