// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  MapRangeIndex.java
//
// Source File Description: 
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M2_B.2  WASX.XD   02/24/2005   newport   new code drop
// 271122       WASX.XD   04/28/2005   kjlaw     add FinderException to finder methods.
// 285430       WASX.XD   06/21/2005   pdykes    Pre-GM changes to remove non-supported public interfaces
// b1153        XD6.1     05/30/2006   cheng1    B25: Indexing enhancement
// b2400        XD6.1     11/22/2006   cheng1    add findRange with includes flags
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.index;

import java.util.Iterator;

import com.ibm.websphere.objectgrid.ObjectMap;

/**
 * This interface is an index that allows comparison type searches for a range of keys.
 *
 * @since WAS XD 6.0.1
 * 
 * @see ObjectMap#getIndex(String)
 */
public interface MapRangeIndex extends MapIndex 
{
    /**
     * Finds all keys with entries with an attribute value greater than the 
     * specified value.
     * 
     * @param attributeValue is the low endpoint of the range excluding the low 
     *        attribute value.
     * 
     * @return the set of keys for the entries with an attribute value greater
     *         than the specified attribute value.
     *           
     * @throws IllegalArgumentException if attributeValue is <code>null</code>.  
     * @throws FinderException if an exception or retry limit is reached when 
     *         concurrent transactions updating the index prevent findGreater 
     *         from completing.
     */
    // TODO should argument be of type Comparable? This applies to all finder methods of this interface.
    Iterator /*<Object key>*/ findGreater(Object attributeValue) throws FinderException;

    /**
     * Find all keys with entries with an attribute value greater than or equal 
     * to the specified value.
     * 
     * @param attributeValue is the low endpoint of the range including the low 
     *        attribute value.
     * 
     * @return the set of keys for the entries with an attribute value greater
     *         than or equal to the specified attribute value. 
     *           
     * @throws IllegalArgumentException if attributeValue is <code>null</code>.  
     * @throws FinderException if an exception or retry limit is reached when 
     *         concurrent transactions updating the index prevent 
     *         findGreaterEqual from completing.
     */
    Iterator/*<Object key>*/ findGreaterEqual(Object attributeValue) throws FinderException;

    /**
     * Finds all keys with entries with an attribute value less than the 
     * specified value.
     * 
     * @param attributeValue is the high endpoint of the range excluding the
     *        high attribute value.
     * 
     * @return the set of keys for the entries with an attribute value less than
     *         the specified attribute value.
     *           
     * @throws IllegalArgumentException if attributeValue is <code>null</code>.  
     * @throws FinderException if an exception or retry limit is reached when 
     *         concurrent transactions updating the index prevent findLess 
     *         from completing.
     */
    Iterator/*<Object key>*/ findLess(Object attributeValue) throws FinderException;

    /**
     * Finds all keys with entries with an attribute value less than or equal to
     * the specified value.
     * 
     * @param attributeValue is the high endpoint of the range including the
     *        high attribute value.
     * 
     * @return the set of keys for the entries with an attribute value less than 
     *         or equals to the specified attribute value
     *          
     * @throws IllegalArgumentException if attributeValue is <code>null</code>.  
     * @throws FinderException if an exception or retry limit is reached when 
     *         concurrent transactions updating the index prevent findLessEqual 
     *         from completing.
     */
    Iterator/*<Object key>*/ findLessEqual(Object attributeValue) throws FinderException;

    /**
     * Returns all keys for the entries with an attribute value inclusively 
     * within the specified range such that 
     * lowAttributeValue <= attribute value < highAttributeValue.
     * 
     * @param lowAttributeValue is the low endpoint of the range including the 
     *        low attribute value.
     * @param highAttributeValue is the high endpoint of range excluding the 
     *        high attribute value.
     *  
     * @return the set of keys for the entries with an attribute value in specified range
     *         of attribute values.
     *          
     * @throws IllegalArgumentException if either lowAttributeValue or highAttributeValue
     *         argument is <code>null</code> or lowAttributeValue > highAttributeValue.  
     * @throws FinderException if an exception or retry limit is reached when 
     *         concurrent transactions updating the index prevent findRange 
     *         from completing.
     */
    Iterator/*<Object key>*/ findRange(Object lowAttributeValue, Object highAttributeValue) throws FinderException;

    /**
     * Returns all keys for the entries with an attribute value.  The includesLowAttributeValue parameter determines whether to include low attribute value. 
     * The includesHighAttributeValue parameter determines whether to include high attribute value. 
     * 
     * @param lowAttributeValue is the low endpoint of the range.
     * @param highAttributeValue is the high endpoint of range.
     * @param includesLowAttributeValue if true, include the low attribute value of the range.
     * @param includesHighAttributeValue if true, include the high attribute value of the range.
     *  
     * @return the set of keys for the entries with an attribute value in specified range
     *         of attribute values.
     *          
     * @throws IllegalArgumentException if either lowAttributeValue or highAttributeValue
     *         argument is <code>null</code> or lowAttributeValue > highAttributeValue.  
     * @throws FinderException if an exception or retry limit is reached when 
     *         concurrent transactions updating the index prevent findRange 
     *         from completing.
     */
    Iterator/*<Object key>*/ findRange(Object lowAttributeValue, Object highAttributeValue, boolean includesLowAttributeValue, boolean includesHighAttributeValue) throws FinderException;

    
    /**
     * Returns the keys for the entries that have the minimum attribute value.
     * 
     * @return the list of the keys for the entries with the minimum attribute value
     * 
     * @throws FinderException if an exception or retry limit is reached
     *         when concurrent transactions updating the index prevent findMin from completing.
     * @since WAS XD 6.1
     */
    Iterator findMin() throws FinderException;
    
    /**
     * Returns the keys for the entries that have the maximum attribute value.
     * 
     * @return the list of the keys for the entries with the maximum attribute value
     * 
     * @throws FinderException if an exception or retry limit is reached
     *         when concurrent transactions updating the index prevent findMax from completing.
     * @since WAS XD 6.1
     */
    Iterator findMax() throws FinderException;
}
