// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  MapSerializer.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io;

import com.ibm.websphere.objectgrid.plugins.BackingMapPlugin;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.MapDataDescriptor;

/**
 * Identifies how data is formatted in a map and how it is serialized.  The MapDataDescriptor
 * describes any relationships between this map an another map.<p>
 *
 * A map can only have one MapSerializerPlugin.  See the {@link SerializerAccessor} for detials
 * on registering a MapSerializerPlugin with a BackingMap.<p>
 *
 * The {@link BasicMapSerializerPlugin} class is a concrete implementation of a MapSerializerPlugin
 * that can be used directly in the ObjectGrid configuration or can be extended to provide
 * custom capabilities.
 *
 * @since 7.1.1
 */
public interface MapSerializerPlugin extends BackingMapPlugin {

    /**
     * The MapDataDescriptor that describes the map.
     * @return the MapDataDescriptor
     */
    public MapDataDescriptor getMapDataDescriptor();

    /**
     * The KeySerializer to use for all key entries in the map.
     * @return the KeySerializer or null if none.
     */
    public KeySerializerPlugin getKeySerializerPlugin();

    /**
     * The ValueSerializer to use for all value entries in the map.
     * @return the ValueSerializer or null if none.
     */
    public ValueSerializerPlugin getValueSerializerPlugin();
}
