// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  Attribute.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io.datadescriptor;


/**
 * Identifies an attribute.  The attribute name and type are required.
 *
 * Use {@link DataDescriptorFactory#createAttribute()} methods to create an Attribute instance.
 * @since 7.1.1
 * @see DataDescriptorFactory
 * @see DataDescriptor
 * @see EmbeddedType
 */
public interface Attribute {

    /**
     * Retrieves the name of the attribute.
     *
     * @return the name of the attribute or null if not yet set.
     */
    public String getAttributeName();

    /**
     * Sets the name of the attribute.<p>
     *
     * Attributes names must include letters, digits and the underscore '_' character.  Spaces and special characters are not permitted.
     *
     * @param attributeName the name of the attribute.  Must not be null.
     * @throws IllegalStateException if the attribute has been initialized.
     */
    public void setAttributeName(String attributeName);

    /**
     * Retrieves the type of the attribute.
     * @return the type of the attribute or null if not yet set.
     */
    public AttributeType getAttributeType();

    /**
     * The type of the attribute.  If the type is {@link AttributeType#EMBEDDED}, then the EmbeddedType must also be set.
     * @param attributeType the type of the attribute. Must not be null.
     * @throws IllegalStateException if the attribute has been initialized.
     */
    public void setAttributeType(AttributeType attributeType);

    /**
     * Retrieves the embedded attribute type.
     * @return the embedded attribute type or null if the type is not an EmbeddedType.
     */
    public EmbeddedType getEmbeddedType();

    /**
     * The type of the embedded attribute.  If set, the AttributeType must be set to {@link AttributeType#EMBEDDED}
     * @param embeddedType the embedded type of the attribute or null if the type is not an EmbeddedType.
     * @throws IllegalStateException if the attribute has been initialized.
     */
    public void setEmbeddedType(EmbeddedType embeddedType);

    /**
     * Returns true if the attribute has zero or more values.
     * @return true if the attribute has zero or more values, false if there is zero or one.
     */
    public boolean isCollection();

    /**
     * Identifies the attribute as a collection attribute values.  Collections of
     * attribute values can have zero or more instances set.
     * @param isCollection answrs true if the attribute is a collection.
     * @throws IllegalStateException if the attribute has been initialized.
     */
    public void setCollection(boolean isCollection);

//	/**
//	 * True if the specified attribute is optional.
//	 * @return
//	 */
//	public boolean isOptional();
//
//	public void setOptional(boolean isOptional);
//
//	public boolean isNullable();
//
//	public void setNullable(boolean isNullable);

}
