// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  MapDataDescriptor.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io.datadescriptor;

import java.util.Map;

import com.ibm.websphere.objectgrid.plugins.Destroyable;
import com.ibm.websphere.objectgrid.plugins.Initializable;
import com.ibm.websphere.objectgrid.plugins.io.MapSerializerPlugin;

/**
 * {@link MapSerializerPlugin} plug-ins create a MapDataDescriptor using the {@link DataDescriptorFactory}
 * to expose any relationships  that it wants to expose to the eXtreme Scale runtime.
 * @since 7.1.1
 */
public interface MapDataDescriptor extends Initializable, Destroyable {

    /**
     * The default name used to identify the key for APIs that refer to maps.
     */
    public String DEFAULT_ADDRESSABLEKEYNAME = "key";


    /**
     * Retrieves the addressable attribute name to use for accessing KeyDataSerializer data when
     * using partial key lookups such as a query.
     * @return the attribute name to use to identify the key, this value is required and must not be null.
     */
    public String getAddressableKeyName();

    /**
     * Identify the addressable attribute name to use for accessing KeyDataSerialier data when
     * using partial key lookups such as a query.  If the addressable name cannot exists in associated
     * {@link ValueDataDescriptor}, the key will have priority.
     * @param addressableKeyName the attribute name to use to identify the key, this value is required and must not be null.
     * @throws IllegalStateException if the descriptor has been initialized.
     */
    public void setAddressableKeyName(String addressableKeyName);


    /**
     * Returns the associations between this map and another map.  Each association
     * identifies a relationship between two maps using a foreign key.<p>
     *
     * If associations are part of an embedded attribute, then the association
     * is represented as a series of Associated types with embedded Associated types.<p>
     *
     * Example:
     * A one-to-one association to the grandchildkey:
     * <pre>
     * parent/child/grandchildkey
     * </pre>
     *
     * Becomes:
     * <pre>
     * OneToOne(name=parent, type=Embedded, tgtMap=GrandChildMap, EmbeddedType=   &LT;--- Path Association
     *   OneToOne(name=child, type=Embedded, tgtMap=GrandChildMap, EmbeddedType=  &LT;--- Path Association
     *     OneToOne(name=grandchildkey, tgtMap=GrandChildMap, type=EmbeddedType=  &LT;--- The Actual Association Key
     *       OneToOne(name=id, type=INT32))))
     * </pre>
     *
     *
     * If the MapDataDescriptor has been initialized, the Map will be read-only.
     * @return the map of Associations
     */
    public Map<String, Association> getAssociations();

    /**
     * Identifies any associations between this map and another map.  Each association
     * identifies a relationship between two maps using a foreign key.
     * @param associations a Map of association names to the Association definition, or null if no associations are defined.
     * @throws IllegalStateException if the descriptor has been initialized.
     */
    public void setAssociations(Map<String, Association> associations);
}
