// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  SerializedEntry.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io.dataobject;

import java.io.Serializable;

import com.ibm.websphere.objectgrid.io.XsDataInputStream;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer.UserReadable;

/**
 * A data object wrapper that wraps a data object and its serialized form.<p>
 *
 * A SerializedEntry implements the {@link java.io.Serializable} interface and can therefore
 * be serialized using Java serialization methods.  Since SerializedEntry's are associated
 * with a {@link DataObjectContext}, serialization of the SerializedEntry will remove
 * the context from the object, and it will become detached.  When detached, methods that require the context will
 * throw an IllegalStateException.  Use the {@link DataObjectContextAware#applyContext(DataObjectContext)} method to
 * reassociate the context with the SerializedEntry.<p>
 *
 * The data represented by this SerializedEntry is created using a {@link DataSerializer}.
 * @since 7.1.1
 */
public interface SerializedEntry extends DataObjectEntry, DataObjectContextAware, Serializable {
    /**
     * Return a user readable representation of the entry.  This is equivalent to
     * using to using a {@link UserReadable}
     * to retrieve the string.<p>
     *
     * If {@link DataObjectContextAware#getContext()} answers null, the result is undefined.
     * @return a user readable representation of the entry.
     */
    public String toString();

    /**
     * Return a data input stream for the serialized form of the entry.
     * <p>
     * Do not store or use the data stream beyond the confines of the
     * method call that receives the SerializedEntry object.
     * The data underlying the stream is owned by the WebSphere eXtreme Scale
     * framework and my be pooled or reused as required.
     *
     * @return the data input stream with read-only access to the serialized form of the entry.
     */
    XsDataInputStream getInputStream();

    /**
     * Return the Object form of the data object. The SerializedEntry, when returned from
     * any eXtreme Scale API will be a copy.  However, subsequent calls to the same
     * SerializedEntry instance will produce the same Object instance.
     * @return the object instance as created by the DataSerializer
     * @throws IllegalStateException if {@link DataObjectContextAware#getContext()} answers null.
     */
    public <T> T getObject();
}
