// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2010
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b21871       XS7.1.1   12/06/2010   pradine   eXtreme Scale OSGi Support
// b21871       XS7.1.1   05/18/2011   pradine   eXtreme Scale OSGi Support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.osgi;

import org.osgi.service.blueprint.container.BlueprintContainer;
import org.osgi.service.blueprint.container.ComponentDefinitionException;
import org.osgi.service.blueprint.container.NoSuchComponentException;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;

/**
 * A built-in BluePrint implementation of the {@link PluginServiceFactory}.
 * <p>
 * Customer can use this built-in {@link BluePrintServiceFactory} to configure a
 * plug-in bean factory.
 * <p>
 * Here is a configuration sample:
 *
 * <pre>
 * {@code
 * <blueprint xmlns="http://www.osgi.org/xmlns/blueprint/v1.0.0">
 *
 *    <!-- Configure the beans -->
 *    <!-- The default scope is singleton -->
 *    <bean id="myProtoBufSerializerBean" class="com.ibm.websphere.samples.xs.serializer.proto.ProtoMapSerializer" scope="prototype">
 *        <property name="keyType" value="com.ibm.websphere.samples.xs.serializer.proto.DataObjects2$OrderKey" />
 *        <property name="valueType" value="com.ibm.websphere.samples.xs.serializer.proto.DataObjects2$Order" />
 *    </bean>
 *
 *
 *    <!-- Configure the factory beans to expose the beans -->
 *    <bean id="myProtoBufSerializer" class="com.ibm.websphere.objectgrid.plugins.osgi.BluePrintServiceFactory">
 *        <property name="blueprintContainer" ref="blueprintContainer"/>
 *        <property name="pluginId" value="myProtoBufSerializerBean"/>
 *    </bean>
 *
 *   <!-- Configure the factory services -->
 *   <service ref="myProtoBufSerializer" interface="com.ibm.websphere.objectgrid.plugins.osgi.PluginServiceFactory"/>
 * </blueprint>
 *
 * }
 * </pre>
 *
 * @since 7.1.1
 */
public class BluePrintServiceFactory implements PluginServiceFactory {
    private final static TraceComponent tc = Tr.register(BluePrintServiceFactory.class.getName(),
            Constants.TR_OSGI_GROUP_NAME, Constants.TR_RESOURCE_BUNDLE_NAME);

    private BlueprintContainer container;

    private String id;

    /**
     * Default Constructor
     */
    public BluePrintServiceFactory() {
    }

    /**
     * Set the BlueprintContainer instance
     * @param container the BlueprintContainer instance
     */
    public void setBlueprintContainer(BlueprintContainer container) {
        this.container = container;
    }

    /**
     * Set the Blueprint bean ID that declares the plugin class.
     * @param id the plugin's bean ID
     */
    public void setBeanId(String id) {
        this.id = id;
    }

    /* (non-Javadoc)
     * @see com.ibm.websphere.objectgrid.plugins.osgi.PluginServiceFactory#getService()
     */
    public Object getService() {
        if (tc.isEntryEnabled())
            Tr.entry(tc, "getService");

        Object service = null;

        try {
            service = container.getComponentInstance(id);
        } catch (NoSuchComponentException nsce) {
            FFDCFilter.processException(nsce, getClass().getName() + ".getService", "110", this, new Object[] {
                    container, id });
            throw nsce;

        } catch (ComponentDefinitionException cde) {
            FFDCFilter.processException(cde, getClass().getName() + ".getService", "115", this, new Object[] { container,
                    id });
            throw cde;
        }

        if (tc.isEntryEnabled())
            Tr.exit(tc, "getService", service);

        return service;
    }
}
