// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.3 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/security/plugins/Authenticator.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:24:14
//
// Module  :  Authenticator.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD601_M2_Bx  WASX.XD   08/15/2005   jitang    ObjectGrid security support
// PK17543      WASX.XD   01/11/2006   jitang    Update JavaDoc
// ------------ --------- ------------ --------- ---------------------------------------
package com.ibm.websphere.objectgrid.security.plugins;

import javax.security.auth.Subject;

/**
 * <p>This plugin can be used to authenticate an ObjectGrid client to an ObjectGrid 
 * server based on the credential provided by the client. A <code>Subject</code>
 * object is returned as a result of authentication.</p> 
 * 
 * <p>This plugin is used in an ObjectGrid server. It can be configured in the 
 * ObjectGrid cluster XML file. </p>
 * 
 * <p>The <code>Credential</code> passed in the <code>authenticate(Credential)</code>
 * method can contain any credential information users desire. For example, it could be a 
 * <code>Credential</code> object containing a user password pair. </p>
 * 
 * <p>ObjectGrid provides several built-in implementations for this interface:
 * <ul>
 * <li>com.ibm.websphere.objectgrid.security.plugins.builtins.CertificateMappingAuthenticator: 
 * An authenticator that simply maps a SSL certficate to a Subject.</li>
 * <li>com.ibm.websphere.objectgrid.security.plugins.builtins.KeyStoreLoginAuthenticator:
 * An authenticator that authenticates a user ID and password to a key file.</li>
 * <li>com.ibm.websphere.objectgrid.security.plugins.builtins.LDAPAuthenticator: 
 * An authenticator that authenticates a user ID and password to a LDAP server.</li>
 * <li>com.ibm.websphere.objectgrid.security.plugins.builtins.WSTokenAuthenticator:
 * An authenticator that authenticates a WebSphere Application Server securty token.</li>
 * </ul>
 * 
 * <P>Refer to the respective JavaDoc for more details.</p>
 * 
 * @since WAS XD 6.0.1
 * 
 * @see Credential
 */
public interface Authenticator {

    /**
     * Authenticates a user represented by the credential object.
     * 
     * @param credential the user Credential
     * 
     * @return a Subject object representing the user
     * 
     * @throws InvalidCredentialException if the credential is invalid
     * @throws ExpiredCredentialException if the credential is expired
     * 
     * @see Credential
     */
    Subject authenticate(Credential credential) 
        throws InvalidCredentialException, ExpiredCredentialException;
}