// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $id$
//
// Module  :  ObjectGridAuthorization.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b4719        WASX.XD   12/21/2006   jian.tang Add security to ObjectGrid
// ------------ ---------- ------------ -------------------------------------------------
package com.ibm.websphere.objectgrid.security.plugins;

import java.security.Permission;

import javax.security.auth.Subject;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.security.AgentPermission;
import com.ibm.websphere.objectgrid.security.MapPermission;
import com.ibm.websphere.objectgrid.security.ObjectGridPermission;
import com.ibm.websphere.objectgrid.security.ServerMapPermission;

/**
 * <p>This plugin can be used to authorize <code>ObjectGrid</code>, <code>ObjectMap</code> and 
 * <code>JavaMap</code> accesses to the <code>Principal</code>s represented 
 * by a <code>Subject</code> object. </p> 
 * 
 * <p>A typical implementation of this plug-in is to retrieve the 
 * Principals from the Subject object, and then check whether or not 
 * the specified permissions are granted to the Principals.
 * </p>
 * <p>A permission passed to the {@link #checkPermission(Subject, Permission)} method 
 *  could be one of the following.:
 * <ul>
 * <li> {@link MapPermission} </li>
 * <li> {@link ObjectGridPermission} </li>
 * <li> {@link AgentPermission} </li>
 * <li> {@link ServerMapPermission} </li>
 * </ul>
 * 
 * @since WAS XD 6.1
 * 
 * @see ObjectGridPermission
 * @see MapPermission
 * @see AgentPermission
 * @see ServerMapPermission
 * @see ObjectGrid#setObjectGridAuthorization(ObjectGridAuthorization)
 */
public interface ObjectGridAuthorization  {

    /**
     * Checks whether or not the <code>Principal</code>s represented by the specified
     * <code>Subject</code> has the specified Permission. 
     * <p>
     * If the permission is granted, <code>true</code> is returned; otherwise 
     * <code>false</code> is returned.
     *   
     * @param subject the subject to check
     * @param permission the permission to access any ObjectGrid resitrcited entities, 
     * such as an ObjectGrid, an ObjectMap, or a JavaMap. The <code>Permission</code> object
     * could be a <code>ObjectGridPermission</code> or a <code>MapPermission</code> object. 
     * 
     * @return <code>true</code> if the permission is granted; <code>false</code> otherwise.
     * 
     * @see Permission
     * @see ObjectGridPermission
     * @see MapPermission
     */
    boolean checkPermission(Subject subject, Permission permission);

}
