// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  SpringLocalTxManager.java
//
// Source File Description:
//
//      See javadoc for this class.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b9449        6.1       08/24/2007   bnewport  new code drop
// b10090       XD 6.1    10/10/2007   cdjohnson Fix javadoc
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.spring;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.Session;

/**
 * This interface has the methods for interacting with the ObjectGrid Spring LocalTransactionManager. It also allows
 * the desired partition to use with this thread to be specified.
 * @see ObjectGridSpringFactory#getLocalPlatformTransactionManager()
 *
 * @since WAS XD 6.1 FIX3, XC10
 */
public interface SpringLocalTxManager {

	/**
	 * This indicates the ObjectGrid to use on this thread when a session is requested.
	 * This replaces any previously associated ObjectGrid, i.e. only a single grid instance can be associated
	 * with a thread at a time.
	 *
	 * @param grid the ObjectGrid to set on this thread.
	 * @see #getSession()
	 */
	void setObjectGridForThread(ObjectGrid grid);

	/**
	 * This returns a managed session for the ObjectGrid associated with this thread.<p>
	 *
	 * Do not call <tt>begin()</tt>, <tt>commit()</tt> or <tt>rollback()</tt> directly
	 * on the session. Spring manages the transaction automatically.
	 *
	 * @return A managed Session to use with this thread.
	 * @throws CannotGetObjectGridSessionException thrown when an ObjectGrid session can't be retrieved.
	 * @see #setObjectGridForThread(ObjectGrid)
	 */
	Session getSession();
}