// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006,2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  MapIndexStatsModule.java
//
// Source File Description:
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 13570         XD6105    12-4-2008   bsandman  code drop
// ------------ --------- ------------ --------- -----------------------------------------


package com.ibm.websphere.objectgrid.stats;

import com.ibm.websphere.objectgrid.TxID;
import com.ibm.websphere.objectgrid.management.HashIndexMBean;
import com.ibm.websphere.objectgrid.plugins.LogSequence;
import com.ibm.websphere.objectgrid.plugins.index.MapIndexPlugin;

/**
 * <p>This interface represents a statistic module for a HashIndex. A HashIndex
 * statistic module contains 6 statistics:
 *
 * <ol>
 * <li>batch update count
 * <li>collision count
 * <li>find operation count
 * <li>find operation duration time
 * <li>failure count
 * <li>find operation result count
 * </ol>
 *
 * The path to a map statistic module is the ObjectGrid name,
 * concatenated with "/", map name, and then the index name. You can also use an array
 * to represent the path, with the first element being the ObjectGrid name,
 * the second element being the map name, and the third being the index name.
 *
 * @since WAS XD 6.1.0.5
 *
 * @see HashIndexMBean#retrieveStatsModule()
 */
public interface HashIndexStatsModule extends StatsModule {

	/**
     * Gets the find operation's invocation count statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the find count statistic
     *
     * @see ActiveCountStatistic
     */

    ActiveCountStatistic getFindCount(boolean copyMode);

    /**
     * Gets the duration time statistic for the find operation.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the duration time statistic
     *
     * @see ActiveTimeStatistic
     */

    TimeStatistic getFindDurationTime(boolean copyMode);

    /**
     * Gets the result set's count statistic for the find operation.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the result count statistic
     *
     * @see ActiveCountStatistic
     */

    ActiveCountStatistic getFindResultCount(boolean copyMode);

    /**
     * Gets the failure count statistic for the find operation.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the failure count statistic
     *
     * @see ActiveCountStatistic
     */

    ActiveCountStatistic getFindFailureCount(boolean copyMode);

    /**
     * Gets the collision count statistic for the find operation.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the collision count statistic
     *
     * @see ActiveCountStatistic
     */

    ActiveCountStatistic getFindCollisionCount(boolean copyMode);

    /**
     * Gets the batch update count statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the doBatchUpdate method's invocation count statistic
     *
     * @see ActiveCountStatistic
     * @see MapIndexPlugin#doBatchUpdate(TxID txid, LogSequence sequence)
     */

    ActiveCountStatistic getBatchUpdateCount(boolean copyMode);

}
