// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.4 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/stats/StatsModule.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:25:24
//
// Module  :  StatsModule.java
//
// Source File Description: 
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD601_M2_B.x WASX.XD   09/07/2005   jitang    ObjectGrid - Statistics enablement for 6.0.1 
// 13570		WASX.XD	  12/04/2008   bsandman	 added in new types
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.stats;

/**
 * This interface represents a statistic module. This is a super class of all the 
 * statistic module interfaces.
 *   
 * @since WAS XD 6.0.1
 * 
 * @see MapStatsModule
 * @see OGStatsModule
 * @see SessionStatsModule
 * @see HashIndexStatsModule
 * @see AgentStatsModule
 * @see QueryStatsModule
 */
public interface StatsModule {

    /** 
     * The path delimiter to separate the paths to a statistic module 
     */
    public static final String PATH_DELIMITER = "/";

    /**
     * Undefined module type 
     */
    public static final int MODULE_TYPE_UNDEFINED = 0;

    /** 
     * Map statistic module type 
     */
    public static final int MODULE_TYPE_MAP = 1;

    /** 
     * ObjectGrid statistic module type 
     */
    public static final int MODULE_TYPE_OBJECT_GRID = 2;

    /**
     * Replication statistic module type 
     */
    // public static final int MODULE_TYPE_REPLICATION = 3;

    /**
     * Server statistic module type 
     */
    // public static final int MODULE_TYPE_SERVER = 4;

    /**
     * Session statistic module type 
     */
    public static final int MODULE_TYPE_SESSION = 5;

    /**
     * HashIndex statistic module type 
     */

    public static final int MODULE_TYPE_HASHINDEX = 6;

    /**
     * Agent statistic module type 
     */
    public static final int MODULE_TYPE_AGENT = 7;

    /**
     * Query statistic module type 
     */
    public static final int MODULE_TYPE_QUERY = 8;

    /**
     * Virtual Memory (disk overflow) module type
     * made private by moving to VMStatsModule for now.
     * Value 9 should still be reserved for VM
     */
    
    //public static final int MODULE_TYPE_VM=9;
    
    /**
     * Gets the <code>StatsSpec</code> object associated with this stats module.
     * <p>
     * Users can know which statistics are enabled or disabled from 
     * this StatsSpec object.
     * 
     * @return the StatsSpec object associated with this stats module
     * 
     * @see StatsSpec
     */
    StatsSpec getStatsSpec();

    /**
     * Resets all the statistics in this module to zero. 
     */
    void reset();

    /**
     * Combines the stats in this stats module with stats from another stats module.
     * 
     * @param m the stats module to be combined
     */
    void combine(StatsModule m);

    /**
     * Sets whether this stats module belongs to a StatsFact or StatsGroup.
     * <p> 
     * If it belongs to a <code>StatsGroup</code>, a <code>true</code> value
     * should be set; otherwise false will be returned.
     * 
     * @param isGroupModule <code>true</code> if this module belongs to a 
     *        StatsGroup; <code>false</code> if this module belongs to a StatsFact
     */
    void setGroupModule(boolean isGroupModule);

    /**
     * Gets a string representation of this statistic module including all the 
     * statistics in it.
     * 
     * @return the string representation of this module's statistics
     */
    String statsToString();

    /**
     * Updates the statistics in this stats module. This will make all the statistics 
     * have the latest value.
     */
    void update();

    /**
     * Gets a copy of this object.
     * 
     * @return a copy of this object.
     */
    StatsModule copy();

    /**
     * Gets the statistic path to this module.
     * <p>
     * A statistic path is an identifier for the statistic module in the 
     * statistic repository. Each statistic module should define its own unique 
     * path. 
     *  
     * @return the stats path to this module.
     */
    String getPath();

}
