// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Source File Description:
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b11208       WASX.XD   04/13/2008   jian.tang Write-behind loader support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.writebehind;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.TTLType;
import com.ibm.websphere.objectgrid.TxID;
import com.ibm.websphere.objectgrid.plugins.LogSequence;

/**
 * This class contains constants used for write behind loaders.
 * <p>
 * If a map is configured with write-behind loader update, the updates (could be insert type,
 * remove type, or update type) to the backend are not instantly updated to the backend by
 * calling the {@link com.ibm.websphere.objectgrid.plugins.Loader#batchUpdate(TxID, LogSequence)}
 * method. Instead, they are queued in a write-behind queue map, and then updated to
 * the backend periodically.
 *
 * @see BackingMap#setWriteBehind(String)
 *
 * @since XD 6.1.0.3
 */
public class WriteBehindLoaderConstants {

    /**
     * A write-behind update is pushed to the backend periodically within a different
     * transaction from the one the update is made to ObjectGrid. When the write-behind update to the
     * backend fails, for example, due to data integrity problem, it is too late to roll back
     * the original ObjectGrid transaction.  ObjectGrid will invalidate the entry and create
     * an entry in a failed database update map.
     * The name of this failed database update map Map is
     * WriteBehindLoaderConstants.WRITE_BEHIND_FAILED_UPDATES_MAP_PREFIX+baseMapName.
     * The key of the entry in this map is an auto-increment Integer, and the value is a
     * {@link com.ibm.websphere.objectgrid.plugins.LogElement}.
     * <p>
     * You can use {@link Session#getMap(String)} to get a reference to the failed database
     * upate map, and then use {@link ObjectMap#getNextKey(long)} to get the next failed
     * update and then take appropriate actions to compensate the failure.
     * <p>
     * We recommend removing the failed update from this failed database update map periodically.
     * By default, this failed database update map is configured with a TTL evictor. The
     * The type of the TTL evictor is {@link TTLType#CREATION_TIME} and the time-to-live is
     * 1 hour.
     * <p>
     * For example, you configure map "payroll" as write-behind. If the write-behind updates
     * to the database fails, all the failed database updates will be stored in the map
     * WriteBehindLoaderConstants.WRITE_BEHIND_FAILED_UPDATES_MAP_PREFIX+"payroll".
     *
     */
    public static final String WRITE_BEHIND_FAILED_UPDATES_MAP_PREFIX = "IBM_WB_FAILED_UPDATES_";

}
