// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  Basic.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b995         6.1       05/22/2006   cdjohnson New part.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.annotations;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * Identifies a field or method as a basic persistable attribute.
 * @since WAS XD 6.1
 */
@Target({FIELD, METHOD})
@Retention(RUNTIME)
public @interface Basic {
    /**
     * The alternative name for this attribute.  The alias can be used to map
     * one entity to another or override the default property name behavior
     * in legacy objects.  It can also be used to refer to this attribute in queries.
     * @return the alias name.
     */
    String alias() default "";
    
//    /**
//     * Determines if the value may be null (for non-primitive types).
//     * @return false if a non-null value must always exist. 
//     */
//    boolean optional() default true;    
    
    /**
     * Suggestion to the projector on how to retrieve the data for this 
     * attribute.<p>
     * 
     * The <tt>EAGER</tt> strategy instructs the projector to to eagerly
     * fetch the data.  The <tt>LAZY</tt> strategy is a <i>hint</i> to the 
     * projector to lazily fetch the data when it is first accessed.<p>
     * 
     * Primitives (values that have no null value) are always fetched eagerly.
     * 
     * @return the FetchType hint.
     */
    FetchType fetch() default FetchType.EAGER;
}
