// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007,2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  EntityListeners.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b9003        XD 6.1.0.1 06/23/2007  jian.tang Entity lifecycle support
// b11518       6.1.0.3   04/08/2008   cdjohnson add invalidate support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.annotations;

import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * Identifies general entity listener classes to receive entity lifecycle callback notifications.<p>
 *
 * The lifecycle callback notifications include:
 * <ul>
 * <li>{@link PrePersist}
 * <li>{@link PostPersist}
 * <li>{@link PreUpdate}
 * <li>{@link PostUpdate}
 * <li>{@link PreRemove}
 * <li>{@link PostRemove}
 * <li>{@link PreInvalidate}
 * <li>{@link PostInvalidate}
 * <li>{@link PostLoad}
 * </ul>
 *
 * This annotation may be applied to an entity class and any number of entity listener classes may be defined
 * on the EntityListeners annotation.  If multiple entity listeners are defined, the callbacks are invoked in the
 * defined order.<p>
 *
 * An entity listener class must be stateless, and have a public, no-arg constructor.  The lifecycle
 * of the entity listener instance is undefined.
 *
 * @since WAS XD 6.1 FIX3
 */
@Target({TYPE}) @Retention(RUNTIME)
public @interface EntityListeners {
	Class[] value();
}