// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  PrePersist.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b9003        XD 6.1.0.1 06/23/2007  jian.tang Entity lifecycle support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.annotations;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * Identifies the pre-persist callback method which is invoked for an entity before the
 * entity has been persisted to the store, which includes entities that have been
 * persisted due to a cascading operation.  This method is run on the thread of the
 * EntityManager.persist operation.<p>
 *
 * This annotation may be applied to an entity class method with the signature:<br>
 * <tt>void &LT;METHOD&GT;()</tt><p>
 *
 * or an entity listener class method with the signature:<br>
 * <tt>void &LT;METHOD&GT;(Object)</tt>.<p>
 *
 * The <tt>Object</tt> argument is the entity instance that the callback method is invoked.
 * It may optionally be declared as the actual entity type.
 * The callback method must not be <tt>static</tt> or <tt>final</tt>.<p>
 *
 * If this callback method throws an unchecked/runtime exception, then any transaction in
 * which this callback is included will be rolled-back.
 *
 * @since WAS XD 6.1 FIX3
 * @see EntityListeners
 */
@Target({METHOD}) @Retention(RUNTIME)
public @interface PrePersist {}