// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  AccessType.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b2203        6.1       08/29/2006   cdjohnson New part
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.md;

import java.io.Serializable;

/**
 * The method in which the persistent state of an entity is accessed.  
 * 
 * @see EntityMetadata
 * @since WAS XD 6.1
 */
public final class AccessType implements Serializable {

    private static final long serialVersionUID = -3423550687884702602L;

    /**
     * The persistent state of the entity object is accessed
     * using fields.
     */
    public final static AccessType FIELD = new AccessType(0);

    /**
     * The persistent state of the entity object is accessed
     * using methods.
     */
    public final static AccessType PROPERTY = new AccessType(1);

    /**
     * The method in which the persistent state of an entity is accessed is not defined.
     */
    public final static AccessType UNDEFINED = new AccessType(2);

    private final static AccessType[] PRIVATE_VALUES = new AccessType[] { FIELD, PROPERTY, UNDEFINED };

    /**
     * Id that uniquely identifies the AccessType instance.
     */
    final private int accessType;

    /**
     * Hidden CTOR for enumerated type idiom to use.
     * 
     * @param accessType
     *            is the unique id for the contructed AccessType instance.
     */
    private AccessType(int accessType) {
        this.accessType = accessType;
    }

    /**
     * Hidden default CTOR for enumerated type idiom to use.
     */
    private AccessType() {
        this(-1);
    }

    /**
     * Returns a string representation of the <code>AccessType</code>.
     * 
     * @return a string representation of the <code>AccessType</code>.
     */
    public String toString() {
        switch (accessType) {
        case 0:
            return "FIELD"; //$NON-NLS-1$
        case 1:
            return "PROPERTY"; //$NON-NLS-1$
        case 2:
            return "UNDEFINED"; //$NON-NLS-1$
        default:
            return "UNKNOWN: " + accessType; //$NON-NLS-1$
        }
    }

    /**
     * Retreive the value of this AccessType.  Can be useful when 
     * using with a switch statement.
     * @return the int value of this AccessType. 
     */
    public int getAccessType() {
        return accessType;
    }

    /*
     * Required to avoid creating a different object instance each time the object is deserialized.
     */
    private Object readResolve() {// throws ObjectStreamException {
        return PRIVATE_VALUES[accessType];
    }

    /**
     * Retrieves the hash code of this AccessType object.
     * @return the hash code value of this object.
     */
    public int hashCode() {
        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + accessType;
        return result;
    }

    /**
     * Returns true if the input object is equivalent to
     * this AccessType.
     * 
     * @param obj the object with which to compare.
     * @return true if the input Object is equivalent to this AccessType
     */
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final AccessType other = (AccessType) obj;
        if (accessType != other.accessType)
            return false;
        return true;
    }
}
