// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006-2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  EntityMetadata.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b995         6.1       05/22/2006   cdjohnson New part.
// b1591        6.1       06/27/2006   cdjohnson M4 feature additions
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.md;


/**
 * The metadata for an entity.  An entity is a collection of data that consists of
 * a set of keys and/or values.<p>
 *
 * Entities can be associated to other entities only if the entity has a key and a name.
 *
 * @since WAS XD 6.1
 */
public interface EntityMetadata {

    /**
     * Get the internally generated identifier for this entity.  Each
     * entity has a unique id within a schema.
     * @return the id of this EntityMetadata
     */
    int getId();

    /**
     * Get the name for this entity.
     * @return the name for this entity or null if this entity is not addressable (anonymous).
     */
    String getName();

    /**
     * The TupleMetadata for the entity's key.  If the key does not contain any attributes, then this
     * entity represents a value-only entity and cannot be the target of an association.
     * @return the TupleMetadata for the entity's key.
     */
    TupleMetadata getKeyMetadata();

    /**
     * The TupleMetadata for the entity's value.  If the value does not contain any attributes or
     * associations, then this entity represents a key-only entity.
     * @return the TupleMetadata for the entity's value.
     */
    TupleMetadata getValueMetadata();

    /**
     * Retrieve the entity class that is associated with this metadata.
     * @return the class that the metadata is associated with or void.class if there is no metadata class.
     */
    Class getMetadataClass();
    

    /**
     * The method in which the persistent state of the entity is accessed by 
     * the runtime.<p>
     * 
     * If the entity has field-based access, the runtime accesses instance variables
     * directly.<p>
     * 
     * If the entity has property-based access, the runtime accesses persistent 
     * state using property accessor methods. 
     * @return the AccessType
     */
    AccessType getAccessType();
    
//    /**
//     * Retrieve custom properties associated with this metadata.
//     * @return the custom properties associated with this metadata.
//     */
//    Properties getProperties();

    /**
     * Return the EntityMetadata for the superset of this EntityMetadata.  
     * @return the superset of this subset EntityMetadata or null if this EntityMetadata is not
     * a subset of another EntityMetadata
     */
    EntityMetadata getSupersetEntityMetadata();
    
    /**
     * Return the EntityMetadata that represents the composite key class metadata for this EntityMetadata. 
     * @return the IdClass that is associated with this entity or null if 
     * no IdClass was specified.
     */
    EntityMetadata getIdClassMetadata();
    
    /**
     * Return true if this EntityMetadata represents the root of the 
     * entity schema.  There can only be one root per schema.
     * @return true if this EntityMetadata is the root of the schema.
     */
    boolean isSchemaRoot();

    /**
     * Retrieve the entity class name that is associated with this metadata.  If there is no class
     * associated with this entity metadata, {@link #getMetadataClass} will return void.class.  This method
     * will return the class name that was specified in the entity descriptor file.
     * <p>
     * If this EntityMetadata has no class associated with it, the name returned will be in the format
     * of "@<class less entity metdata identifier>".
     * @return the class name or the class less entity metadata identifier.
     * @since 7.0.0.0 FIX2
     */
    String getMetadataClassName();

}
