// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006-2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  TupleAssociation.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b995         6.1       05/22/2006   cdjohnson New part.
// b1591        6.1       07/26/2006   cdjohnson M4 feature additions
// b2404        6.1       10/12/2006   rajeshk   M6 feature additions - cascade
// b2405        6.1       11/15/2006   jian.tang support lazy association fetch type
// b11518       6.1.0.3   04/08/2008   cdjohnson add invalidate support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.md;

/**
 * The metadata for an association between two entities.<p>
 *
 * @since WAS XD 6.1
 */
public interface TupleAssociation extends TupleAttribute
{
	/**
	 * The cardinality for this end of an association.
	 * @return the AssociationType
	 */
	AssociationType getAssociationType();

	/**
	 * The EntityMetadata for the target of this association
	 * @return the EntityMetadata for the associated entity
	 */
	EntityMetadata getTargetEntityMetadata();

    /**
     * The name of the target entity
     * @return the name of the target entity.
     */
    String getTargetEntityName();

    /**
     * The attribute name of the owner of a bi-directional association.
     * @return the name of the owning side of a bi-directional association.
     */
    String getMappedByAssociationName();

    /**
     * Identifies the persistent operations that should be cascaded to
     * the target entity.
     * @return CascadeInfo associated with this association
     */
    CascadeInfo getCascadeInfo();

    /**
     * Identifies how to fetch the associated entity from the persistent store.
     * @return the fetch type for this association
     */
    FetchType getFetchType();

    /**
     * Identifies how to order a multi-valued association.
     * @return the OrderByInfo for this association or null if not present.
     */
    OrderByInfo getOrderByInfo();

    /**
     * Ordering information for multi-valued associations.
     * @since WAS XD 6.1
     * @see com.ibm.websphere.projector.annotations.OrderBy
     */
    interface OrderByInfo {
        /**
         * The order-by string in the format:<p>
         * <pre>
         * orderby_list::= orderby_item [,orderby_item]*
         * orderby_item::= property_or_field_name [ASC | DESC]
         * </pre><p>
         *
         * @return the order-by string.
         */
        String getOrderByString();

        /**
         * The elements used to order the association collection.
         * @return the ordered elements to order by.
         */
        OrderByElement[] getOrderByElements();

        /**
         * Ordering details for an attribute in the target entity.
         * @since WAS XD 6.1
         */
        interface OrderByElement {
            /**
             * The name of the field to order.
             * @return the field name
             */
            String getFieldName();

            /**
             * If true, order the collection in ascending order based on the field name.  If false
             * order in descending order.
             * @return true if ascending ordering.
             */
            boolean isAscending();
        }
    }

    /**
     * Cascade information for entity association persistence.
     * @since WAS XD 6.1
     * @see com.ibm.websphere.projector.annotations.CascadeType
     */
    interface CascadeInfo {
        /**
         * If true, cascade persist operations to the target entity.
         * @return true if persist should be cascaded.
         */
        boolean isCascadePersist();

        /**
         * If true, cascade remove operations to the target entity.
         * @return true if remove should be cascaded.
         */
        boolean isCascadeRemove();

        /**
         * If true, cascade merge operations to the target entity.
         * @return true if merge should be cascaded.
         */
        boolean isCascadeMerge();

        /**
         * If true, cascade refresh operations to the target entity.
         * @return true if refresh should be cascaded.
         */
        boolean isCascadeRefresh();

        /**
         * If true, cascade invalidate operations to the target entity.
         * @return true if invalidate  should be cascaded.
         * @since WAS XD 6.1.0.3
         */
        boolean isCascadeInvalidate();
    }
}
