//IBM Confidential OCO Source Material
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
//The source code for this program is not published or otherwise divested
//of its trade secrets, irrespective of what has been deposited with the
//U.S. Copyright Office.
//
//Source File Description:
//
//    See javadoc.
//
//Change Activity:
//
//Reason       Version   Date         Userid    Change Description
//------------ --------- ------------ --------- -----------------------------------------
// wi906       8.5       02/29/2012   cdjohnson new part
//------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.xs.ra;

import javax.resource.NotSupportedException;
import javax.resource.ResourceException;
import javax.resource.cci.ConnectionMetaData;
import javax.resource.cci.Interaction;
import javax.resource.cci.LocalTransaction;
import javax.resource.cci.ResultSetInfo;

import com.ibm.websphere.objectgrid.Session;
import com.ibm.ws.xs.ra.cci.XSConImpl;

/**
 * An XSConnection represents an application level connection handle to an eXtreme Scale ObjectGrid.
 * 
 * @since 8.5
 * @since XC10
 */
public final class XSConnection 
implements javax.resource.cci.Connection {

	private XSConImpl xsCon = null;
	
	/**
	 * Retrieve the ObjectGrid Session associated with this connection.
	 * @return The ObjectGrid Session
	 */
	public Session getSession() throws ResourceException {
		return xsCon.getSession();
	}
	
	/**
	 * Initiates close of the connection handle at the application level.
	 * @see javax.resource.cci.Connection#close()
	 * @throws ResourceException
	 */
	public void close() throws ResourceException {
		xsCon.close();
	}
	
	/**
	 * Creates an Interaction associated with this Connection. 
	 * @return Interaction instance
	 * @see javax.resource.cci.Connection#createInteraction
	 * @throws ResourceException
	 * @throws NotSupportedException
	 */
	public Interaction createInteraction() throws ResourceException {
		return xsCon.createInteraction();
	}

	/**
	 * Returns an LocalTransaction instance that enables a component to demarcate 
	 * local transactions on the Connection. 
	 * @return LocalTransaction instance
	 * @see javax.resource.cci.Connection#getLocalTransaction()
	 * @throws ResourceException
	 */
	public LocalTransaction getLocalTransaction() throws ResourceException {
		return xsCon.getLocalTransaction();
	}

	/**
	 * Gets the information on the underlying eXtreme Scale instance represented through an active connection.
	 * @return WXS ConnectionMetaData
	 * @see javax.resource.cci.Connection#getMetaData()
	 * @throws ResourceException 
	 */
	public ConnectionMetaData getMetaData() throws ResourceException {
		return xsCon.getMetaData();
	}

	/**
	 * Gets the information on the ResultSet functionality supported by a connected eXtreme Scale instance.
	 * @return ResultSetInfo instance
	 * @see javax.resource.cci.Connection#getResultSetInfo()
	 * @throws ResourceException 
	 */
	public ResultSetInfo getResultSetInfo() throws ResourceException {
		return xsCon.getResultSetInfo();
	}
	
    /**
     * Warning: This method is used via reflection. Do not change its signature without updating the reflection code.
     * The reflection lookup is called from:
     *   - com.ibm.ws.xs.ra.spi.XSMConImpl.getConnection()
     */
	@SuppressWarnings("unused")
	private void setImpl(XSConImpl con) {
		this.xsCon = con;
	}

	/**
	 * Warning: This method is used via reflection. Do not change its signature without updating the reflection code.
	 * * The reflection lookup is called from:
     *   - com.ibm.ws.xs.ra.spi.XSMConImpl.associateConnection()
	 */
    @SuppressWarnings("unused")
	private XSConImpl getImpl() {
		return xsCon;
	}
    
    public String toString() {
        return super.toString() + "[" + (xsCon == null ? "none" : xsCon.toString() ) + "]";   
    }
}
