// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005-2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  SessionIDOverride.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.xs.sessionmanager;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

/**
 * This interface can be implemented by WXS Http session replication users who
 * want to control the user session identifier. Any implementation will be
 * instantiated with a no argument constructor. 
 *
 * @since 8.6
 */
public interface SessionIDOverride {
    
    /**
     * An implementation of this interface will be passed in to all 
     * <code>SessionIDOverride.getID(SessionIDContext)</code> calls so the
     * <code>SessionIDOverride</code> implementation can determine the unique
     * session identifier for the end user
     *
     * @since 8.6
     */
    public interface SessionIDContext {
        /**
         * <p>This method will be called by the 
         * <code>SessionIDOverride.getID(SessionIDContext)</code>
         * implementation to retrieve the <code>HttpServletRequest</code>
         * object associated with the request. The implementation can
         * determine the unique session identifier based on data in this
         * object.
         * @return the HttpServletRequest object.
         */
        HttpServletRequest getRequest();
    }
    
    /**
     * An implementation of this interface will be passed in to all 
     * <code>SessionIDOverride.init(InitializationContext)</code> calls so the
     * <code>SessionIDOverride</code> implementation can be initialized
     *
     * @since 8.6
     */
    public interface InitializationContext {
        /**
         * <p>This method will be called by the 
         * <code>SessionIDOverride.init(InitializationContext)</code>
         * implementation to retrieve the <code>ServletContext</code>
         * object associated with the web application. The implementation is free
         * to cache any data that may be used to later determine the unique
         * session identifier.
         * @return the servlet context
         */
        ServletContext getServletContext();
    }
    
    /**
     * <p>This method will be called by the WXS Http session client when a web
     * application is first initialized, typically the first request to a web
     * application. The <code>InitializationContext</code>
     * is passed in for the user to cache anything that may be of use when
     * the getID method is subsequently called.
     * 
     * @param ctx the context parameters to be used to initialize the plug-in. 
     */
    public void init(InitializationContext ctx);
    
    /**
     * <p>This method will be called by the WXS Http session client when a web
     * application is destroyed, typically when a web application is stopped.
     * The user can cleanup any data they may have cached at this time.
     * 
     */
    public void destroy();
    
    /**
     * <p>Returns a session identifier based on the <code>SessionIDContext</code>.</p>
     *
     * <p>This method will be called by the WXS Http session client to determine
     * the identifier that will be used to store the session data in the grid.
     *
     * <p>In order to use this non-default option, the application must set a cookie
     * or have some other method to uniquely identify end users via the HttpServletRequest.
     * 
     * <p>This method will be called by the WXS Http session client rather than the standard
     * HttpSession.getId() API. If the method returns null, the session will be stored in
     * the data grid with the default HttpSession.getId() method.
     * 
     * @param ctx the context parameters to be used to retrieve the session identifier. 
     * @return the unique user session identifier
     *
     */
    public String getID(SessionIDContext ctx);
}
